/* -*-C-*-
 *
 * Based on (from ebsa110/devconf.h):
 * $Revision: 1.1.1.1 $
 *   $Author: drusling $
 *     $Date: 1998/09/22 15:48:17 $
 *
 * Copyright (c) 1996 Advanced RISC Machines Limited.
 * All Rights Reserved.
 *
 *   Project: ANGEL
 *
 *     Title: Hardware description of the EB285 board.
 */

#ifndef angel_eb285_devconf_h
#define angel_eb285_devconf_h

#include "stacks.h"

#define CACHE_SUPPORTED    1
#define PCMCIA_SUPPORTED   0
#define PARALLEL_SUPPORTED 0
#define PROFILE_SUPPORTED  0
#define PCI_SUPPORTED      1
#define SYSTEM_COPROCESSOR_SUPPORTED	1

#define ENABLE_MMU         1
#define ENABLE_CLOCK_SWITCHING 1


/* If USE_RARP is 0 then an IP address must be supplied of the form
        __IP1.__IP2.__IP2.__IP4
        (it is set in arm_ether.c) */
#define USE_RARP 0
#define USE_BOOTP 1

#if (USE_RARP == 1 && USE_BOOTP == 1 )
#error "Can't RARP and BOOTP - see"  __file__
#endif

#if ((USE_RARP == 0) && (USE_BOOTP == 0)) // The hardwired IP address

#define __IP1    16
#define __IP2    36
#define __IP3     0
#define __IP4   184

#endif

/*
 * Choose the method for debug output here.  Options supported for PIE are:
 *      panicblk        panic string written to RAM
 *      pidulate        via ARMulator (during emualtion only)
 *      logadp          via ADP to host
 */
#ifdef DEBUG
# define DEBUG_METHOD logterm
#endif

/*
 * ring buffer size
 *
 * NOTE - the ring buffer macros are more efficient
 * when ring buffer size is declared as a power of 2
 */
#define RING_SIZE (512)

#define RING_LOW_WATER_MARK   (32)             /* when to fill tx buffer */
#define RING_HIGH_WATER_MARK  (RING_SIZE-32)  /* when to empty rx buffer */

/*************************************************************************/
/**                                                                     **/
/**     These defines determine whether Angel uses FIQs or IRQs         **/
/**                                                                     **/
/*************************************************************************/
#define HANDLE_INTERRUPTS_ON_IRQ 0
#define HANDLE_INTERRUPTS_ON_FIQ 1

/* See serlock.h for the possible settings */
#define FIQ_SAFETYLEVEL FIQ_CannotBeOptimised

/* control whether the serial device is raw or Angel/shared */
#if !defined(MINIMAL_ANGEL) || MINIMAL_ANGEL == 0
#  define RAW_EBS_SERIAL 0      /* change to 1 if should always be raw */
#else
#  define RAW_EBS_SERIAL 1
#endif

/* the default active device */
#if (ETHERNET_SUPPORTED == 1)
#define DE_DEFAULT_ACTIVE DI_ETHERNET
#else
#define DE_DEFAULT_ACTIVE DI_SERIAL
#endif

#include "platform.h"

/* The EBSA285 memory map is structured as follows. */

#define RAMBase         (DRAM_BASE)              /* normal RAM location */
#define RAMSize         (PCI_DRAMBASE)                          

#define RAM2Base        (PCI_DRAMBASE)              /* Makelo.c requires this */
#define RAM2Size        (0x00000000)                          

#define ROMBase         (0x41000000)              /* ROM/FLASH etc. */
#define ROMSize         (0x01000000)                          

#define FLASHBase       (0x41000000)              /* ROM/FLASH etc. */
#define FLASHSize       (0x01000000)                          

#define IOBase          (0x40000000)
#define IOSize          (0x00040000)                          

#define PCIBase          (0x80000000)
#define PCISize          (0x80000000)                          

/*
 * The ARM 7 PIE and EBSA110 allow unrestricted access to the entire memory space.
 * To stop the debugger acessing areas it should these two macros are used
 * to determine if an access should occur.

 * The ARM 6 PIE card prohibits user mode code from reading the I/O space.
 * So these macros are superfluous in that case.
*/
#define READ_PERMITTED(__addr__) ((__addr__) < PCIBase)
#define WRITE_PERMITTED(__addr__) ((__addr__) < PCIBase)

/* This can be overridden by the memory sizer - if present */
#define Angel_DefaultTopOfMemory (RAMSize)

/* This should be set to 1 for systems with RAM slots */
#define MEMORY_SIZE_MAY_CHANGE_DYNAMICALLY 0


/*
 * the sizes of the various Angel stacks
 */
#define Angel_UNDStackSize      0x0100
#define Angel_ABTStackSize      0x0100

#define Angel_AngelStackSize    0x2000
#define Angel_SVCStackSize      0x1000

# if HANDLE_INTERRUPTS_ON_IRQ
#  define Angel_IRQStackSize     0x0800
# else
#  define Angel_IRQStackSize     0x0100
# endif

# if HANDLE_INTERRUPTS_ON_FIQ
#  define Angel_FIQStackSize     0x0800
# else
#  define Angel_FIQStackSize     0x0100
# endif

/* for the fatal error handler (only!) */
#define Angel_FatalStackSize    0x0400

/* This is set to 1 only if the system does not remap ROM to 0
 * on a reset
 */
#define BRANCH_TO_HIGH_ROM_ON_RESET 0

/* Determine whether Angel will run from ROM or RAM - this
 * should be as close to the top of memory as possible
 */
#if (ROADDR > Angel_DefaultTopOfMemory) 
#else
# define Angel_InRAMAddr ROADDR
#endif

/* We need some space for Page tables - put them at the top of memory
 * above the Application stack and the Angel stacks (by default).
 */

/*                -----------------     Top of Memory
 * Ethernet Table |               |
 *                -----------------
 * Level2 Table   |               |
 *                -----------------
 * Level1 Table   |               |
 *                -----------------     Angel_ApplStackOffset
 *                |               |
 *                -----------------     Angel_ApplStackLimitOffset
 * Angel Stacks   |               |
 *                -----------------
 * Angel Angel    |               |
 *                -----------------
 */

#define Ethernet_Table_Size (1<<15)
#define Level1_Table_Size (1<<14)
#define Level2_Table_Size (1<<14) 

#if (ETHERNET_SUPPORTED == 1)
#define Angel_PageTableSize (Ethernet_Table_Size+Level1_Table_Size+Level2_Table_Size)
#else
#define Angel_PageTableSize (Level1_Table_Size+Level2_Table_Size)
#endif


#define Angel_PageTableAreaBaseOffsetFromTopOfMemory (Angel_PageTableSize)

/* For now this assumes that memory is never resized 
 *      (and the force is with you)
 */
 
#define Level1tab           (Angel_DefaultTopOfMemory - Angel_PageTableSize)
#define Level2tab_ROM       (Level1tab + Level1_Table_Size)
#define EthernetQBufferBase (Level2tab_ROM + Level2_Table_Size)


/* And where should the Angel stacks live ?
 * They can either live at an absolute address (eg the bottom of memory)
 * or somewhere relative to the top of memory (which is dynamic!), so
 * this must be specified here.  You may change this if you require
 * a different memory map.
 */
#define Angel_StacksAreRelativeToTopOfMemory 1


#if Angel_StacksAreRelativeToTopOfMemory
# define Angel_StackBaseOffsetFromTopMemory  (-(Angel_PageTableSize + Angel_CombinedAngelStackSize))
# define Angel_ApplStackOffset     Angel_StackBaseOffsetFromTopMemory
#else
# define Angel_FixedStackBase      0x5400
# define Angel_ApplStackOffset     (-Angel_PageTableSize)
#endif

/*
 * The application stack will sit at the top of memory, below the
 * Tables, but above the Angel stacks
 * Stacks grow towards 0x0000
 */

#define Angel_ApplStackSize        8192
#define Angel_ApplStackLimitOffset (Angel_ApplStackOffset - Angel_ApplStackSize)

/*
 * By default the heap starts at the end of the ROM image and the heap
 * limit is the bottom of the application stack.  These are the
 * defaults unless these two #defines are turned on in which case the
 * addresses specified are used by the C Library.
 */
#if 0
# define Angel_ApplHeapLimit  ???
# define Angel_ApplHeap       ???
#endif

/*
 * where to download a new agent to
 */
#define Angel_DownloadAgentArea        0x8000

/*
 * This constant should be changed only with great care.  It is the
 * amount of empty stack space left between two callback's stacks on
 * the AngelStack.
 */
#define Angel_AngelStackFreeSpace 0x400

/*
        Keep this value low enough to fit in one packet
        
*/
#define ETHERNET_BUFFERLONGSIZE 1400 
 
/*---------------------------------------------------------------------------*/
/*-- I/O memory map ---------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

#define IO_base_internal        (IOBase + 0x00000000)


/*
 * the devices available
 * ORDER AND NUMBER MUST MATCH angel_Device[] TABLE IN devices.c
 */
typedef enum DeviceIdent
{
  DI_SERIAL = 0,
#if (ETHERNET_SUPPORTED == 1)
  DI_ETHERNET,
#endif
  DI_NUM_DEVICES
} DeviceIdent;

/* the device which controls the LED's */
#define DI_LED_DEVICE DI_SERIAL

/* 
 * the Interrupt handlers available 
 * ORDER AND NUMBER MUST MATCH angel_IntHandler[] in devices.c
 */
typedef enum IntHandlerID
{
    IH_SERIAL = 0
} IntHandlerID;

#define DE_NUM_INT_HANDLERS 1   /* must be no. of entries in IntHandlerID */


/* 
 * the POLL handlers available 
 * ORDER AND NUMBER MUST MATCH angel_PollHandler[] in devices.c
 */
#if (ETHERNET_SUPPORTED == 1)
typedef enum PollHandlerID       /* Must correspond with devices.c */
{
        PH_ETHERNET=0
} PollHandlerID;
#endif

#define DE_NUM_POLL_HANDLERS ETHERNET_SUPPORTED

#endif

 /* EOF devconf_h */

