;/***********************************************************************
;* Copyright  Intel Corporation 1998.  All rights reserved.
;************************************************************************
; UART registers

; Receive, Transmit, Interupt enable, Interrupt Identification and FIFO
; are only accessable when the Divisor latch access bit in the line
; control register is 0
Rx              EQU     0x0     ; Receive port, read only
Tx              EQU     0x0     ; Transmit port, write only
IntEnable       EQU     0x4     ; Interrupt enable, read/write
IntId           EQU     0x8     ; Interrupt id, read only
FIFOcntl        EQU     0x8     ; FIFO control, write only

        ; With the Divisor latch access bit set the first 2 registers set
        ; the divisor, which controls the line speed.
Dllsb           EQU     0x0
Dlmsb           EQU     0x4

        ; The remaining registers are always accessable and read/write
LineCntl        EQU     0xC     ; Line control, the main control register
ModemCntl       EQU     0x10    ; Modem control
LineStatus      EQU     0x14
ModemStatus     EQU     0x18

; Masks etc. for useful fields

        ; Interrupt enable fields (interrupt enable register)
ERxAvailIntMsk  EQU     0x1     ; Data avail and FIFO timeout interrupts
ETxEmptyIntMsk  EQU     0x2     ; Tx done interrupt
ERxErrorIntMsk  EQU     0x4     ; Rx error interrupts
EModemIntMsk    EQU     0x8     ; Modem status interrupt

        ; Interrupt id values (Interrupt Id register)
IntIdMsk        EQU     0x0F    ; Mask for bits that give interrupt types
NoSerialInt     EQU     0x1     ; No interrupt outstanding
RxErrInt        EQU     0x6     ; Receive Error
RxAvailInt      EQU     0x4     ; Data available or sufficient data in FIFO
RxTimeoutInt    EQU     0x0c    ; Timeout with some data in FIFO
TxEmptyInt      EQU     0x2     ; Tx FIFO empty or send done
ModemInt        EQU     0x0     ; Modem state change

        ; FIFO bits (Interrupt Id register)
FIFOrunningMsk  EQU     0xC0    ; Mask for FIFO bits, both 0 if not in use
                                ; 1 if in use.

        ; FIFO control register bits
FIFOEnableMsk   EQU     0x01    ; Bit 0 enables FIFOs
FIFORxClrMsk    EQU     0x02    ; Bit 1 clears Rx FIFO
FIFOTxClrMsk    EQU     0x04    ; Bit 2 clears Tx FIFO
FIFOLvlMsk      EQU     0x0c0   ; The top two bits set the Rx trigger level
FIFOLvl1        EQU     0x0     ; trigger on 1 byte
FIFOLvl4        EQU     0x40    ; 4 bytes
FIFOLvl6        EQU     0x80    ; 6 bytes
FIFOLvl14       EQU     0x0C0   ; 14 bytes

        ; Line control register bits
WordLenMsk      EQU     0x03    ; Mask for data length
StopBitsMsk     EQU     0x04    ; Number of stop bits
ParityMsk       EQU     0x08    ; Parity enable
ParityTypeMsk   EQU     0x30    ; Even and Stick parity select
SetBreakMsk     EQU     0x40    ; Set break bit, break sent while set
DLABMsk         EQU     0x80    ; Divisor latch.

        ; Word length values
WordLen5        EQU     0x0
WordLen6        EQU     0x1
WordLen7        EQU     0x2
WordLen8        EQU     0x3

        ; Modem control register
ModemCDTRMsk    EQU     0x1
ModemCRTSMsk    EQU     0x2
ModemINTENMsk   EQU     0x8
ModemCLoopMsk   EQU     0x10

        ; Line status register
LineDRMsk       EQU     0x1     ; Data ready
LineOEMsk       EQU     0x2     ; Overrun Error
LinePEMsk       EQU     0x4     ; Parity Error
LineFEMsk       EQU     0x8     ; Framing Error
LineBIMsk       EQU     0x10    ; Break interrupt signal
LineTHREMsk     EQU     0x20    ; Transmitter holding register empty (i.e. ready for next char)
LineTEMTMsk     EQU     0x40    ; Transmitter empty (inc. FIFO if in use)
LineRFEMsk      EQU     0x80    ; Receive FIFO error

        ; Modem status register
ModemSDCSR      EQU     0x1     ; Clear to send changed since last read
ModemSDDSR      EQU     0x2     ; Data set ready changes since last read
ModemSTERI      EQU     0x4     ; End of ring since last read
ModemSDCR       EQU     0x8     ; Carrier detect changed since last read
ModemSCSR       EQU     0x10    ; Clear to send state
ModemSDSR       EQU     0x20    ; Data set ready state
ModemSRI        EQU     0x40    ; Ring indicator state
ModemSCR        EQU     0x80    ; Carrier detect state

        ; Useful line speed values for divider. The internal clock is
        ; 1.8462 Mhz

Baud9600low     EQU     12
Baud9600high    EQU     0

Baud19200low    EQU     6
Baud19200high   EQU     0

Baud38400low    EQU     3
Baud38400high   EQU     0

        END
