/***********************************************************************
 * Copyright  Intel Corporation 1998.  All rights reserved.
 ************************************************************************

  Defines, Structures, Routines used & defined in StrongARM library.

        $Id: system.h,v 1.2 1999/01/26 14:12:27 mquinn Exp $

 ************************************************************************/

#ifndef __system_h      /* Only include stuff once */
#define __system_h

#ifndef __cdefs_h       /* uHAL 'C' type definitions */
#include "cdefs.h"
#endif

#ifndef __address_map_h /* platform specific stuff (address maps etc) */        
#include "platform.h"
#endif

#ifndef __pci_h         /* basic PCI definitions */
#include "pci.h"
#endif

/* These defines are in a common coding practices header file */
#ifndef ERROR
#define ERROR           -1      /* -MAX_INT would be better?? (or errno.h) */
#endif
#ifndef OK
#define OK              0       /* Can never work out should this be 1 or 0? */
#endif
#ifndef FALSE
#define FALSE           0
#endif
#ifndef TRUE
#define TRUE            1
#endif
#ifndef NULL
#define NULL    0
#endif

/* General ERROR/SUCCESSFUL returns */
#define MSG_FAILURE                -1
#define MSG_SUCCESS                 0

/* PCI errors */
#define MSG_ERROR_BAD_BUS        0x80 
#define MSG_ERROR_BAD_DEVICE     0x81 
#define MSG_ERROR_BAD_FUNC       0x82
#define MSG_ERROR_BAD_OFFSET     0x83

#define RESETV          0
#define UNDV            1
#define SWIV            2
#define IABTV           3
#define DABTV           4
#define IRQV            6
#define FIQV            7

/*
 * A device node.  All devices in the system are represented by a devnode.
 */
#define MAX_DEVNODE 10
typedef struct devNodeStruct {
        struct devNodeStruct    *next ;
        U32                      id;
        U32                      handle;
        U32                      intDma;
        /* Store the size requested by each PCI BAR, because cannot read this
         * easily after initialisation.
         * NOTE: Keep the romSize straight after the memSize array.
         */
        U32                      memSize[PCI_MAX_BAR + 1];
        U32                      romSize ;
} devNodeType;

#define FIXED_TYPE      0x00000000
#define PCI_TYPE        0x10000000
#define XBUS_TYPE       0x20000000

/* A function with no argument returning pointer to a void function */
typedef void (*PrVoid)(void) ;
typedef void (*PrHandler)(U32) ;        /* As PrVoid with one parameter */

/* A function with no argument returning PrVoid */
typedef PrVoid (*PrPrVoid)(void) ;

struct SAis_IRQ {
        PrHandler        handler ;      /* Routine for specific interrupt */
        U32              flags ;
        U32              mask ;
        const U8        *name ;         /* Debug, owner id */
        struct SAis_IRQ *next ;         /* Handy for shared interrupts */
};

 
/* Enum to describe timer: free, one-shot, on-going interval or locked-out */
enum    SAe_TimerState  { T_FREE, T_ONESHOT, T_INTERVAL, T_LOCKED } ;

/* Sometimes intervals are longer than the timer can handle:
 *      ticks is no of ticks in a period
 *      ticksleft is no left until trigger
 *      tvalue1 is loaded to timer if ticksleft > 0
 *      tvalue2 is loaded to timer if ticksleft == 0
 *              so ticks = period / MAX_PERIOD
 *                 ticksleft = ticks
 *                 tvalue1 = period / ticks
 *                 tvalue2 = period % ticks
 */
struct SAis_Timer {
        U32                      irq ;          /* IRQ number */
        enum SAe_TimerState      state ;
        U32                      period ;       /* Period between triggers */
        U32                      ticks ;
        U32                      ticksleft ;
        U32                      expire ;
        U32                      timer1 ;
        U32                      timer2 ;
        PrHandler                handler;       /* User Routine */
        const U8                *name;          /* Debug, owner id */
        struct SAis_Timer       *next;
} ;

typedef struct {
        U32 memSize;
        U32 memType;
        U32 cpuId;
        U32 platformId;
        U8  res[4];
} infoType, *pInfoType;

/*
 * Function declarations 
 */

/* defined in boot.s */
extern void     *SAr_StartOfRam(void) ;
extern void     *SAr_EndOfFreeRam(void) ;
extern void     *SAr_EndOfRam(void) ;
extern void     SAr_TrapSWI(void) ;
extern void     SAr_InitBSSMemory(void) ;

/* defined in irq.c */
extern void     SAr_DefineIRQ(PrVoid, PrPrVoid, PrVoid) ;
extern void     SAr_InitIRQ(void) ;
extern S32      SAr_RequestIRQ(U32, PrHandler, const U8 *) ;
extern S32      SAr_FreeIRQ(U32) ;
extern void     SAr_EnableIRQ(U32) ;
extern void     SAr_DisableIRQ(U32) ;
extern void     SAir_UnexpectedIRQ(U32) ;
extern void     SAr_DispatchIRQ(U32 flags) ;


/* defined in lib/<blah>/platform.c */
extern S32      SAr_WriteLED(U32, U32) ;
extern void     SAir_MaskIrq(U32 irq) ;
extern void     SAir_UnmaskIrq(U32 irq) ;
extern void     SAr_GetPlatformInfo(pInfoType p) ;
extern void     SAir_AddFixedDevNodes(void) ;
#if uHAL_PCI != 0
extern U32      SAir_PciMakeConfigAddress(U32, U32) ;
#endif
extern U32      SAr_PciHost(void) ;
extern void     SAir_PlatformDisableTimer(U32 timer) ;
extern void     SAir_PlatformEnableTimer(U32 timer) ;

/* PCI routines are in pci.h which is included earlier in this file */

/* defined in irqtrap.s */
extern void     SAr_TrapIRQ(void) ;
extern void     SAr_ComplexTrapIRQ(void) ;
extern PrVoid   SAr_NewIRQ(PrHandler, PrVoid) ;

/* defined in control.s */
extern void     cpuControlWrite(U32);
extern U32      cpuControlRead(void);
extern U32      cpuIdRead(void);

/* defined in irqlib.s */
extern U32      SAir_EnterSvcMode(void) ;
extern U32      SAir_EnterLockedSvcMode(void) ;
extern void     SAir_ExitSvcMode(U32 spsr) ;

extern void     SAir_DisableInt(void) ;
extern void     SAir_EnableInt(void) ;

extern U32      SAir_ReadMode(void) ;
extern void     SAir_WriteMode(U32) ;

extern U32      SAir_DoSWI(U32, U32) ;

/* defined in crt.c */
extern void     printf(char *, ...) ;           /* Variable args */

extern void     *memset(char *s, int c, int n) ;
extern void     *memcpy(char *s, char *ct, int n) ;
extern int      memcmp(char *cs, char *ct, int n) ;
extern int      strlen(const char *s) ;

/* defined in led.c */
extern void     SAr_SetLED(U32) ;
extern void     SAr_ResetLED(U32) ;
extern S32      SAr_ReadLED(U32) ;
extern U32      SAr_CountLEDs(void) ;
extern U32      SAr_InitLEDs(void) ;

/* defined in timer.c */
extern void     SAr_InitTimers(void) ;
extern S32      SAr_RequestTimer(PrHandler, const U8 *) ;
extern void     SAr_InstallTimer(U32) ;
extern S32      SAr_RequestSystemTimer(PrHandler, const U8 *) ;
extern void     SAr_EnableTimer(U32) ;
extern S32      SAr_FreeTimer(U32) ;

extern S32      SAr_GetTimerState(U32) ;
extern S32      SAr_SetTimerState(U32, enum SAe_TimerState) ;
extern S32      SAr_GetTimerInterval(U32) ;
extern S32      SAr_SetTimerInterval(U32, U32) ;

extern U32      SAir_GetSystemTimer(void) ;
extern U32      SAir_CountTimers(void) ;
extern void     SAir_DisableTimer(U32) ;
extern S32      SAir_GetTimerIRQ(U32) ;

/* Aliases for direct system access */
#define SAr_InstallSystemTimer()          SAr_InstallTimer(OS_TIMER)
#define SAir_GetSystemTimerState()        SAr_GetTimerState(OS_TIMER)
#define SAir_SetSystemTimerState(state)   SAr_SetTimerState(OS_TIMER, state)
#define SAir_GetSystemTimerInterval()     SAr_GetTimerInterval(OS_TIMER)
#define SAir_SetSystemTimerInterval(time) SAr_SetTimerInterval(OS_TIMER, time)
#define SAir_GetSystemTimerIRQ()          SAir_GetTimerIRQ(OS_TIMER)
#define SAir_EnableSystemTimer()          SAr_EnableTimer(OS_TIMER)

/* defined in iolib.c */
extern int      outb(unsigned char b, void *p) ;
extern int      inb(void *p) ;
extern void     outw(U16 w, void *p) ;
extern U32      inw(void *p) ;
extern void     outl(U32 l, void *p) ;
extern U32      inl(void *p) ;

extern void     putchar(char) ;
extern int      getchar(void) ;
extern void     PutString(char *) ;
extern void     SAr_ResetPort(void) ;

/* defined in cache.c */
extern void     SAr_EnableCache(void) ;
extern void     SAr_DisableCache(void) ;

extern void     SAir_EnableICache(void) ;
extern void     SAir_EnableDCache(void) ;
extern void     SAir_EnableWriteBuffer(void) ;

/* defined in mmu.s */
extern void     SAr_InitMMU(int mode) ;
extern void     SAr_ResetMMU(void) ;

extern void     SAir_WriteCacheMode(int) ;
extern int      SAir_ReadCacheMode(void) ;

extern void     SAir_DisableICache(void) ;
extern void     SAir_DisableDCacheAndWriteBuffer(void) ;

/* defined in devnode.c */
extern S8       SAr_BuildDevNodeTree(void) ;
extern void     SAr_AddDevNode(U32 type, U32 id, U32 handle, U32 intDma) ;
extern U32      SAr_GetDevNodeHandle(U32 id, U32 index) ;
extern void     SAr_PrintDevNodeTree(void) ;
extern devNodeType *SAr_FindDevNode(U32 handle);

/* Alias to find host devNode */
#define SAr_FindHostNode()      SAr_FindDevNode(SAr_PciHost())
#endif
