
	SUBT	EB285 board description and Angel support > ebsa285/target.s

;/***********************************************************************
;* Copyright  Intel Corporation 1998.  All rights reserved.
;************************************************************************
;
; This file defines the EB285 manifests and macros required to support
; the Angel world. It is not a complete description of the board, only
; the information necessary for Angel. It is envisaged that individual
; device drivers or application code will hold the necessary definitions
; for the other board features.
;
; NOTE: To help keep the main Angel source as generic as possible, heavy
; use is made of macros. However, it is likely that the use of macros
; will hide some very obvious and simple optimisations. If optimised
; Angel source is required, then a fixed target version should be created
; by the porting developer. The aim of Angel is to be as simple and quick
; to port as possible.
;
;       $Id$
;
;************************************************************************/

 IF :DEF: OPT
	ASSERT  (listopts_s)
old_opt SETA	{OPT}
	OPT	(opt_off)	; disable listing of include files
 ENDIF

	; ---------------------------------------------------------------------

	IF :LNOT: :DEF: ebsa285_s

		GBLL	ebsa285_s
ebsa285_s	SETL	{TRUE}

	; ---------------------------------------------------------------------
	; The "ROMonly" build variable controls whether the system
	; being built supports a system where only ROM is mapped at
	; zero, and RAM cannot be mapped there. i.e. where the ARM
	; vectors are hard-wired in ROM. We create this variable
	; un-conditionally, to ensure that if an attempt is made to
	; define it elsewhere then a build error will occur.
	
		GBLL	ROMonly	; depends on target
ROMonly		SETL	{FALSE}	; {TRUE} if no RAM at zero


	INCLUDE platform.s
	GET	mmumacro.s

	; ---------------------------------------------------------------------
	; -- ANGEL and uHAL Support macros ------------------------------------
	; ---------------------------------------------------------------------

	; ---------------------------------------------------------------------
	; UNMAPROM
	; --------
	; Provide code to deal with mapping the reset ROM away from zero
	; (if required).

	MACRO
$label  UNMAPROM	$w1,$w2
	MEND

	; ---------------------------------------------------------------------
	; STARTUPCODE
	; -----------
	; Provide code to deal with the main act of mapping the reset
	; ROM away from zero (if required), and ensuring that the
	; target I/O world is suitably clean before attempting to
	; initialise the rest of Angel.
	;
	; NOTE:  do not use r4, that's $pos.
	;
	
	MACRO
$label  STARTUPCODE	$w1,$w2,$pos,$ramsize
	
	LDR	pc, =HiReset 
	; not as silly as it looks ! The ROM is about to disappear
	; from low memory so we have to make sure that we are executing
	; from the ROM's high memory alias.
HiReset
	;
	; Switch memory maps on EBSA110 by writing to anywhere (0 is safe)
	; 
	STR	$w1,[$w1,-$w1]

 IF ENABLE_CLOCK_SWITCHING = 1
	; turn on clock switching
	WRCP15_EnableClockSW $w1
 ENDIF

	; NOTE:	Need to check if X-BUS or PCI-Arbiter is active (Rev 2).
	;	Enable X-BUS to read links/switches.
	INIT_XBUS	$w1, $w2

 IF {ENDIAN} = "big"
	; Need to be in big-endian mode before any byte/half-word accesses
	MOV	$w1, #EnableBigEndian
	WRCP15_Control	$w1	; Update control register
 ENDIF

	; Okay, now initialise LEDs to a known state - off
	MOV	$w1, #0			; all off
	
	SET_LEDS	$w1, $w2

	; Wait around to allow led pattern to be seen
	LDR	r0, =0x8000
1
	SUBS	r0, r0, #1
	BNE	%B01

  IF :DEF: FLASH_BOOT_LOADER
	; Check which flash block is to be executed (and switch to it).
	BOOTLD	$w1, $w2

	; We only return here if flash block 0 was selected.
  ENDIF		; FLASH_BOOT_LOADER

	; Initialise the DRAM (if present)
	; NOTE: These macros have become so large that startrom.s cannot
	;	handle local branches around STARTUPCODE. So we need to put
	;	in an LTORG _inside_ this macro (with associated branch).
	;vvvvvvvvvvvvvvv
	B	StartMem

	LTORG		; Allows code using STARTUPCODE to do local loads

StartMem
	INIT_RAM	$w1, $w2, $ramsize

	; Finally, tell PCI that initialisation is complete
	MOV	r7, #PCI_DRAMSIZE
	SETUP_PCI	$w1, $w2, r6, r7


	DISABLE_INTS	$w1, $w2

	; Enable the defaults
	LDR	$w2, =CSR_BASE
	LDR	$w1, =ANGEL_INTS
; 	STR	$w1, [$w2, #IRQ_ENABLE_SET]
	STR	$w1, [$w2, #FIQ_ENABLE_SET]

	MOV	$w1, #1				; 001, --Y
	SET_LEDS	$w1, $w2

	MEND

	; ---------------------------------------------------------------------
	; SETUP_RAM
	; ---------
	; ANGEL and uHAL macro to setup memory timing/size registers in 21285
	; on startup. 21285 only has 1 timing register, so DIMMs must work with
	; same timing. Each array is then configured for no. of banks & size.
	;	$w1 -> returns the size of memory allocated
	;	$w4 -> byte array of of ADDR_SIZE masks (33221100)
	;	$w5 -> byte array of offsets between arrays (max 64 * 1MB)
	;	$s2, $s3, $s6 - $s8 -> scratch registers.

	MACRO
	SETUP_RAM	$w1, $s2, $s3, $w4, $w5, $s6, $s7, $s8
	MOV	$w1, #DRAM_BASE		; Start of RAM
	LDR	$s3, =CSR_BASE
	; This enables the test for offset == 0 to be a valid completion test.
	ADD	$w5, $w5, #0x80000000

	LDR	$s2, =SETUP_TIMING
	STR	$s2, [$s3, #DRAM_TIMING]	; Default timing

	ADD	$s3, $s3, #DRAM_ADDR_SIZE_0	; -> 1st ADDR_SIZE reg.
1
	AND	$s6, $w5, #0x3f		; Read next array offset
	MOV	$s2, $w4, LSR $s6	; Shuffle register value down
	AND	$s2, $s2, #0xff		; Register value (- base addr)
	ANDS	$s8, $s2, #0x7		; Mask array size & test for empty

	; When mask is empty, array is disabled
	BEQ	%F02

	; Else, convert bit pattern to array size = (1 << ($s8 - 1))
	; Only increment memory if the mask is not empty
	SUB	$s8, $s8, #1
	MOV	$s7, #1
	MOV	$s7, $s7, LSL $s8
	MOV	$s7, $s7, LSL #20	; array size * 1M
	ADD	$s2, $s2, $w1		; add base of this array
	ADD	$w1, $w1, $s7		; Offset to next array

	; If Array size is 64MB and mode is 0, must use 32MB to
	; differentiate correct size. Last bit of size has to be zero
	; to allow for aliasing (using bit18 on row ma[9]).
	CMP	$s2, #0x7		; Note: no bits for mode 0
	SUBEQ	$s2, $s2, #1

2
	; Writing zero into the ADDR_SIZE register disables this array
	MOV	$s6, $s6, LSR #1	; Offset is twice register offset
	STR	$s2, [$s3, $s6]		; Set up 21285 register
	;STR	$s2, [$s3], #4		; Set up 21285 register
	MOVS	$w5, $w5, LSR #8	; Get next ADDR_SIZE byte offset
	BNE	%B01			; Until done

	MEND

	; ---------------------------------------------------------------------
	; SIZE_RAM
	; ---------
	; ANGEL and uHAL macro to scan memory to evaluate size. Assumes that
	; there are gaps between arrays so that memory will return invalid
	; data (either from a gap or mirrored memory).
	;	$w4 -> step size for memory test
	;	$w5 -> number of steps (max array size / step size)
	;	    -> returns memory size found as multiple of no. of steps
	;	$w1 -> address of top of current array (max possible)
	;	    -> returns address of bottom of current array
	;	$w2, $w3, $w6 -> scratch registers.

	MACRO
	SIZE_RAM	$w1, $w2, $w3, $w4, $w5, $w6
	MOV	$w6, $w5
	ADD	$w1, $w1, #0x3c00
1
	SUB	$w1, $w1, $w4
	SUBS	$w5, $w5, #1		; Countdown 64 -> 0 write
	STR	$w5, [$w1]		; footprint in memory
	BGT	%B1

	MOV	$w3, $w1		; Pointer to base_ram
	STR	$w5, [$w3, #4]		; Shouldn't need to worry about
2					; floating data bus, but..
	LDR	$w2, [$w3]		; Now check looking for unexpected
	SUBS	$w2, $w2, $w5		; memory contents
	BNE	%F3			; End of array found
	ADD	$w3, $w3, $w4
	ADD	$w5, $w5, #1		; Count up 0 -> 64 read
	CMP	$w5, $w6
	BLT	%B2
3
	MEND

	; ---------------------------------------------------------------------
	; RAM_REG
	; -------
	; ANGEL and uHAL macro to convert the memory size into a size mask for
	; the DRAM_ADDR_SIZE registers. Looks for the LSB (since each array is
	; a power of 2 in size) and returns the number of zero's bits which is
	; the mask..
	;	$w1 -> size of array (multiple of 1MB)
	;	$w2 -> returns mask value

	MACRO
	RAM_REG	$w1, $w2		; Convert to DRAM_ADDR_SIZE mask
	MOV	$w2, #0
	CMP	$w1, $w2
	BEQ	%F02			; Empty - nothing to do
1
	ADD	$w2, $w2, #1
	TST	$w1, #1			; LSB set?
	MOV	$w1, $w1, LSR #1
	BEQ	%B01			; No, try next shuffled down bit
2
	MEND

	; ------------------------------------------------------------------
	; GOTO_ROM
	; --------
	; ANGEL and uHAL macro to switch from 0 to high alias of ROM. Some
	; systems cannot just switch to High ROM, but must set up the MMU.

	MACRO
	GOTO_ROM	$w1, $w2

        ; Again, more instructions which should have no effect in ROM-only
        ; systems. Loading pc with SAip_HiReset makes sure we're running from
        ; real rom, rather than a shadowed address. The 1st write after
        ; reset switches the memory map for normal operation.
        LDR     pc,=SAip_HiReset
SAip_HiReset
        STR     r0,[r0,-r0]

	MEND

	; ---------------------------------------------------------------------
	; INIT_RAM
	; --------
	; ANGEL and uHAL macro to initialise memory on startup. Compatability
	; with Angel specifies a1, a2 and v2 as calling registers and that a1
	; must not be used.
	; WARNING: This macro uses lots of registers!
	;	$w1 (a1), $w2 (a2), $w3 (v2), v3, v4, v5, v6, a3 and a4

	MACRO
	INIT_RAM	$w1, $w2, $w3

	; With such pressure on registers, have to make sure the calling
	; registers are just so!
	ASSERT  ( $w1 = r0 )
	ASSERT  ( $w2 = r1 )
	ASSERT  ( $w3 = r5 )

	; Can't initialise RAM if already running from it!!
	; So, need to put a check here
	LDR	$w3, =CSR_BASE
	LDR	$w2, [$w3, #DRAM_TIMING]
	LDR	$w3, =SETUP_TIMING		; Already active ?
	CMP	$w2, $w3
	BEQ	%F29			; Yes, return default size

	; Not setup, but if memory is already active in some other mode,
	; need to disable refresh before re-initialising.
	LDR	$w3, =CSR_BASE
	MOV	$w2, #0
	STR	$w2, [$w3, #DRAM_TIMING]

	; If refresh is just happening, wait 50 clock cycles until it expires
	MOV	$w1, #50
11
	SUBS	$w1, $w1, #2
	BGT	%B11			; Until done

	; Read from each Mode Register array to enable precharge
	MOV	$w1, #MAX_SDRAM
	MOV	$w2, #DC21285_DRAM_A0MR
1
	LDR	$w3, [$w2]		; Just read, don't care about data
	ADD	$w2, $w2, #SD_OFFSET	; Next
	SUBS	$w1, $w1, #1
	BGT	%B01			; Until done

	; Now write to the Mode registers to set the operational mode of the
	; SDRAM. MUST be done to all banks, even if not fitted, to stop the
	; 21285 forcing DQM High, which it does after reset.
	MOV	$w1, #MAX_SDRAM
	MOV	$w2, #DC21285_DRAM_A0MR
2
	STR	$w2, [$w2, #CAS_OFFSET]	; It's the ADDRESS which is important
	ADD	$w2, $w2, #SD_OFFSET	; Next
	SUBS	$w1, $w1, #1
	BGT	%B02			; Until done
	
	; Setup Dram timing & DISABLE all arrays by default.
	;
	; Setting the refresh interval to be minimum & enabling refresh.
	; After (upto) 8 refresh cycles the memory will be ready to use, each
	; refresh cycle is 32 instruction cycles.
	LDR	$w3, =CSR_BASE
	LDR	$w2, =INITIAL_TIMING
	MOV	$w1, #DRAM_TIMING
	STR	$w2, [$w3, $w1]
	MOV	$w2, #MAX_SDRAM
	MOV	$w2, $w2, LSL #2
	ADD	$w1, $w1, $w2		; Registers are after timing register
	MOV	$w2, #0			; Disable each array & move to address 0
3
	STR	$w2, [$w3, $w1]
	SUBS	$w1, $w1, #4
	BGT	%B03			; Until done

	; Wait around until memory is active
	MOV	$w1, #REFRESH_WAIT	; Number of refresh cycles to wait
4
	MOV	$w2, #REFRESH_COUNT	; Length of each cycle
5
	SUBS	$w2, $w2, #2		; 2 instructions in this loop
	BGT	%B05			; Until cycle complete

	SUBS	$w1, $w1, #1		; Another refresh cycle gone..
	BGT	%B04			; Until done

	LDR	v5, =(SDSIZE_64M + MUX_MODE2)
	; v3 holds 4 bytes each containing the bottom 8 bits of the
	; DRAM_ADDR_SIZE registers: all set to biggest SDRAM part.
	ADD	v3, v5, v5, LSL #8	; 0x00bb <- 0x00b0 + 0x000b
	ADD	v3, v3, v3, LSL #16	; 0xbbbb <- 0xbb00 + 0x00bb

	; 00 -> 08 -> 16 -> 24, shifts are done in this order
	LDR	v4, =0x18100800

	; Fix the default SDRAM registers
	SETUP_RAM	$w1, $w2, $w3, v3, v4, v5, v6, a3

; First test each array to see if any memory present. If no memory is found,
; invalidate the appropriate byte of v3 by blanking it out. When memory is
; found, try to determine Mux mode.
	MOV	v3, #0			; Blank result
	MOV	a3, #MAX_SDRAM
	ADD	v5, $w1, #0xf000	; Pointer to max_ram + 1
	LDR	$w2, =0x12345678	; Footprint
6
	SUB	v5, v5, #SZ_64M		; Base of next ram array
	STR	$w2, [v5]
	MVN	$w2, $w2		; Invert

	; This inversion guarantees that the data bus isn't left floating
	; with our test footprint when there is no memory present.
	STR	$w2, [v5, #4]
	MVN	$w2, $w2		; Back to original
	LDR	$w3, [v5]
	SUBS	$w3, $w3, $w2		; Did it write?

	; No, clear mask, but leave as 64M to make other memory tests easier.
	LDRNE	$w3, =SDSIZE_64M
	BNE	%F07

	; Yes some found, now test for different memory layouts:
	; Having initially set all arrays to Mode 2, the address lines are
	; driven as if for that array. So test bits correspond to that mode.

	; Address line A21 distinguishes MUX_MODE0. Write inverse footprint
	; at v5 + A21 - if it is written at v5, array is MUX_MODE0.
	MVN	$w2, $w2		; Now look for holes with invert
	ADD	$w3, v5, #0x00200000	; Add Bit21
	STR	$w2, [$w3]
	MVN	$w2, $w2		; Back to original
	LDR	$w3, [v5]
	SUBS	$w3, $w3, $w2		; Is original footprint still there?
	; No, Mode 0 array
	LDRNE	$w3, =(SDSIZE_64M + MUX_MODE0)
	BNE	%F07

	; Address line A23 distinguishes MUX_MODE2. Write inverse footprint
	; at v5 + A23 - if it is written at v5, array is _not_ MUX_MODE2.
	STR	$w2, [v5]
	MVN	$w2, $w2		; Now look for holes with invert
	ADD	$w3, v5, #0x00800000	; Add Bit23
	STR	$w2, [$w3]
	MVN	$w2, $w2		; Back to original
	LDR	$w3, [v5]
	SUBS	$w3, $w3, $w2		; Is original footprint still there?
	; Yes, Mode 2
	LDREQ	$w3, =(SDSIZE_64M + MUX_MODE2)
	BEQ	%F07

	; Address line A22 distinguishes MUX_MODE4. Write inverse footprint
	; at v5 + A22 - if it is written at v5, array is _not_ MUX_MODE4.
	STR	$w2, [v5]
	MVN	$w2, $w2		; Now look for holes with invert
	ADD	$w3, v5, #0x00400000	; Add Bit22
	STR	$w2, [$w3]
	MVN	$w2, $w2		; Back to original
	LDR	$w3, [v5]
	SUBS	$w3, $w3, $w2		; Is original footprint still there?
	; Yes, Mode 4
	LDREQ	$w3, =(SDSIZE_64M + MUX_MODE4)
	BEQ	%F07

	; Mode 1 or Mode 3?
	; Address line A24 can find most MUX_MODE1 Dimms, since it drives A22
	; on Mode 1 Dimms, but smaller size arrays will still mirror at
	; v5 + A22 (4MB). Best hope is to sort most of the Mode 1s and then
	; catch 'odd' sized Mode 3s.

	STR	$w2, [v5]
	MVN	$w2, $w2		; Now look for holes with invert
	ADD	$w3, v5, #0x01000000	; Add Bit24
	STR	$w2, [$w3]
	MVN	$w2, $w2		; Back to original
	LDR	$w3, [v5]
	SUBS	$w3, $w3, $w2		; Is original footprint still there?
	; Yes, Mode 1 array
	LDREQ	$w3, =(SDSIZE_64M + MUX_MODE1)
	; No, Mode 3 array (probably)
	LDRNE	$w3, =(SDSIZE_64M + MUX_MODE3)

7
	; Okay, add this bank to the mask..
	ADD	v3, $w3, v3, LSL #8
	SUBS	a3, a3, #1
	BGT	%B06			; Repeat until done

	MOV	a4, v3			; Copy array masks

	; 00 -> 08 -> 16 -> 24, shifts are done in this order
	LDR	v4, =0x18100800

	; Fix the default SDRAM registers
	SETUP_RAM	$w1, $w2, $w3, v3, v4, v5, v6, a3

	; Top bit means the test for offset == 0 is a valid completion test.
	LDR	a3, =0x80081018
	LDR	v6, =0			; Store for array of sizes found
8
	LDR	v3, =SZ_1M		; Test memory in 1MB steps
9
	MOV	v4, #64			; i.e. 64 steps per array

	SIZE_RAM	$w1, $w2, $w3, v3, v4, v5

	; Memory sized for this bank
	AND	v5, a3, #0x3f		; Read next array offset (no end flag)
	MOV	$w2, a4, LSR v5		; Current DRAM_ADDR_SIZE mask
	AND	$w2, $w2, #MUX_MASK
	CMP	$w2, #MUX_MODE3		; Check for Mode 3
	BNE	%F10
	CMP	v4, #8			; Can only be 8MB!
	BEQ	%F10

	; Anything which gets here was a badly configured Mode 1 array. Clear
	; the bad mode byte & set array to mode 1. Then do size test again.
	LDR	v4, =0xFF		; Mask out the Mode 3 byte
	MOV	v4, v4, LSL v5
	BIC	a4, a4, v4
	LDR	v4, =(SDSIZE_64M + MUX_MODE1)
	MOV	v4, v4, LSL v5
	ADD	a4, a4, v4		; Put new Mode 1 byte in its place
	MOV	v4, v4, LSR v5		; Back to Mode 1, unshifted

	; Reset the mode for this bank..
	LDR	v3, =(CSR_BASE + DRAM_ADDR_SIZE_0)
	MOV	v5, v5, LSR #1		; Offset is twice register offset
	LDR	$w2, [v3, v5]		; Read SIZE_ADDR
	BIC	$w2, $w2, #0xFF		; Only interested in base offset
	ADD	$w2, $w2, v4		; new mode info
	STR	$w2, [v3, v5]		; Store back in SIZE_ADDR
	MOV	v5, v5, LSL #1		; Restore offset
	ADD	$w1, $w1, #SZ_64M	; Move back to top of this array

	B	%B08			; Do this array again as Mode 1

10
	; Size determined, add to array.
	MOV	v4, v4, LSL v5		; Shuffle upto correct byte
	ADD	v6, v6, v4		; Add the size of this array
	MOVS	a3, a3, LSR #8		; Down to next byte (8 bits) & array
	BNE	%B09			; Until done

; v6 now contains the size of each memory array in multiples of 1MB. This is
; very similar to the DRAM_ADDR_SIZE register array, so convert to that format
; by determining least significant bit for each array.
; Since each array must start naturally aligned with the size of the array,
; the only way to have contiguous memory is to have the arrays sorted into
; size order - largest first.
	LDR	a3, =0x18100800		; Normal array order.
	AND	v5, v6, #0xff
	RAM_REG	v5, $w1			; Get DRAM_ADDR_SIZE mask for array 1
	MOV	v6, v6, LSR #8
	ADD	v6, v6, $w1, LSL #24	; Replace size with mask
	AND	v5, v6, #0xff
	RAM_REG	v5, $w2			; Get mask for array 2
	MOV	v6, v6, LSR #8
	ADD	v6, v6, $w2, LSL #24	; Replace size with mask
	AND	v5, v6, #0xff
	RAM_REG	v5, $w3			; Get mask for array 3
	MOV	v6, v6, LSR #8
	ADD	v6, v6, $w3, LSL #24	; Replace size with mask
	AND	v5, v6, #0xff
	RAM_REG	v5, v4			; Get mask for array 4
	MOV	v6, v6, LSR #8
	ADD	v6, v6, v4, LSL #24	; Replace size with mask

	; Simple bubble sort with additional byte swap in array-order register
	CMP	$w1, $w2		; array 1 < array 2?
	BGE	%F21
	MOV	a4, $w1			; Yes, swap sizes 1 & 2
	MOV	$w1, $w2
	MOV	$w2, a4
	AND	v3, a3, #0xff		; Now swap offsets 1 & 2
	AND	v5, a3, #0xff00
	MOV	a3, a3, LSR #16		; clear offsets 1 & 2
	MOV	a3, a3, LSL #16
	ADD	a3, a3, v3, LSL #8	; offset 1 -> offset 2
	ADD	a3, a3, v5, LSR #8	; offset 2 -> offset 1
21
	CMP	$w1, $w3		; array 1 < array 3?
	BGE	%F22
	MOV	a4, $w1			; Yes, swap sizes 1 & 3
	MOV	$w1, $w3
	MOV	$w3, a4
	AND	v3, a3, #0xff		; Now swap offsets 1 & 3
	AND	v5, a3, #0xff0000
	BIC	a3, a3, #0xff0000	; clear offsets 1 & 3
	BIC	a3, a3, #0xff
	ADD	a3, a3, v5, LSR #16	; offset 3 -> offset 1
	ADD	a3, a3, v3, LSL #16	; offset 1 -> offset 3
22
	CMP	$w1, v4			; array 1 < array 4?
	BGE	%F23
	; Note: just put size 1 in size 4, since the 1 value isn't used.
	MOV	v4, $w1
	AND	v3, a3, #0xff		; Now swap offsets 1 & 4
	AND	v5, a3, #0xff000000
	BIC	a3, a3, #0xff000000	; clear offsets 1 & 4
	BIC	a3, a3, #0xff
	ADD	a3, a3, v5, LSR #24	; offset 4 -> offset 1
	ADD	a3, a3, v3, LSL #24	; offset 1 -> offset 4
23
	CMP	$w2, $w3		; array 2 < array 3?
	BGE	%F24
	MOV	a4, $w2			; Yes, swap sizes 2 & 3
	MOV	$w2, $w3
	MOV	$w3, a4
	AND	v3, a3, #0xff00		; Now swap offsets 2 & 3
	AND	v5, a3, #0xff0000
	BIC	a3, a3, #0xff0000	; clear offsets 2 & 3
	BIC	a3, a3, #0xff00
	ADD	a3, a3, v5, LSR #8	; offset 3 -> offset 2
	ADD	a3, a3, v3, LSL #8	; offset 2 -> offset 3
24
	CMP	$w2, v4			; array 2 < array 4?
	BGE	%F25
	; Note: just put size 2 in size 4, since the 2 value isn't used.
	MOV	v4, $w2
	AND	v3, a3, #0xff00		; Now swap offsets 2 & 4
	AND	v5, a3, #0xff000000
	BIC	a3, a3, #0xff000000	; clear offsets 2 & 4
	BIC	a3, a3, #0xff00
	ADD	a3, a3, v5, LSR #16	; offset 4 -> offset 2
	ADD	a3, a3, v3, LSL #16	; offset 2 -> offset 4
25
	CMP	$w3, v4			; array 3 < array 4?
	BGE	%F26
	; Note: don't need to swap sizes 3 & 4, since the values aren't used.
	AND	v3, a3, #0xff0000	; Now swap offsets 3 & 4
	AND	v5, a3, #0xff000000
	BIC	a3, a3, #0xff000000	; clear offsets 3 & 4
	BIC	a3, a3, #0xff0000
	ADD	a3, a3, v3, LSL #8	; offset 3 -> offset 4
	ADD	a3, a3, v5, LSR #8	; offset 4 -> offset 3
26
	; Read previously set MUX_MODE & BANKS from 21285
	LDR	$w3, =(CSR_BASE + DRAM_ADDR_SIZE_3) ; -> last ADDR_SIZE reg.
	LDR	v3, [$w3], #-4		; Current array 4 ADDR_SIZE
	AND	v3, v3, #MUX_MASK	; Strip out good mux mode/bank value
	LDR	v5, [$w3], #-4		; Current array 3 ADDR_SIZE
	AND	v5, v5, #MUX_MASK
	ADD	v3, v5, v3, LSL #8	; v3 = v5 + (v3 << 8);
	LDR	v5, [$w3], #-4		; Current array 2 ADDR_SIZE
	AND	v5, v5, #MUX_MASK
	ADD	v3, v5, v3, LSL #8	; v3 = v5 + (v3 << 8);
	LDR	v5, [$w3]		; Current array 1 ADDR_SIZE
	AND	v5, v5, #MUX_MASK
	ADD	v3, v5, v3, LSL #8	; v3 = v5 + (v3 << 8);
	ADD	v3, v3, v6		; Add size masks to modes

	MOV	v4, a3			; Byte order needs to be in v4

	; Finally, fix the 21285 SDRAM registers for calculated sizes
	SETUP_RAM	$w1, $w2, $w3, v3, v4, v5, v6, a3

29
	; set up the return arguments.
	MOV	$w3, $w1

	MEND

	; ---------------------------------------------------------------------
	; SETUP_PCI
	; ---------
	; Setup the PCI map to a sensible setup for master or slave.
	; Expects size of memory block in $w4

	MACRO
$label	SETUP_PCI	$w1, $w2, $w3, $w4

	IMPORT	SAiv_PciSizes
	IMPORT	SAiv_PciMasks

	LDR	$w1, =CSR_BASE

	; Disable PCI Outbound interrupts
	LDR	$w2, =NO_OUTBOUND_INTS
	STR	$w2, [$w1, #PCI_OUT_INT_MASK]

	MOV	$w2, #0
	; Disable Doorbells
	STR	$w2, [$w1, #DBELL_PCI_MASK]
	STR	$w2, [$w1, #DBELL_SA_MASK]

	; High PCI address bits all map to 0
	STR	$w2, [$w1, #PCI_ADDR_EXT]

	; Set Interrupt ID to 1 - fixes problems on systems which cannot
	; handle interrupt 0.
	MOV	$w2, #SETUP_PCI_INT_ID
	STR	$w2, [$w1, #PCI_INT_LINE]
	
	; Remove PCI_reset, won't hurt if it's already been done
	LDR	$w2, [$w1, #SA_CONTROL]
	ORR	$w2, $w2, #PCI_NRESET
	STR	$w2, [$w1, #SA_CONTROL]

	; Now, we have to find the correct mask to make the given
	; block of memory available to PCI.
	LDR	$w1, =SAiv_PciSizes
	MOV	$w2, #0
1
	; While there are still entries in the size table..
	LDR	$w3, [$w1, $w2]
	TEQ	$w3, #0
	BEQ	%F02

	; If we're still smaller than this entry, try the next
	CMP	$w3, $w4
	ADDHI	$w2, $w2, #4
	BHI	%B01
2
	; Done, read the mask
	LDR	$w1, =SAiv_PciMasks
	LDR	$w2, [$w1, $w2]

	; ..and write it the the config register
	LDR	$w1, =CSR_BASE
	STR	$w2,[$w1, #DRAM_BASE_ADDR_MASK]

	; PCI can access memory from here..
	LDR	$w2, =PCI_DRAMBASE
	STR	$w2, [$w1, #DRAM_BASE_ADDR_OFF]

	; Don't init PCI if central function & Standalone Bench Bit is set
	LDR	$w2, [$w1, #SA_CONTROL]
	TST	$w2, #PCI_CFN
	BEQ	%F03			; Not central function

	LDR	$w2, =IMAGE_SWITCH
	LDR	$w2, [$w2]
	TST	$w2, #BENCH_BIT
	BNE	%F03

	; Don't respond to any commands
	MOV	$w2, #0
	STR	$w2, [$w1, #PCI_COMMAND]

	STR	$w2, [$w1, #PCI_DRAM_BAR]
	; Set the Memory Base Address Register to our default
	LDR	$w2, =DC21285_DRAM_A0MR
	STR	$w2, [$w1, #PCI_MEM_BAR]
	MOV	$w2, #0xf000
	STR	$w2, [$w1, #PCI_IO_BAR]

	; Done: respond to I/O space & Memory transactions.
	;	ALSO BE PCI MASTER
	LDR	$w2, =PCI_CFN_INIT
	STR	$w2, [$w1, #PCI_COMMAND]

3
	; Signal PCI_init_complete, won't hurt if it's already been done
	LDR	$w2, [$w1, #SA_CONTROL]
	ORR	$w2, $w2, #INIT_COMPLETE
	STR	$w2, [$w1, #SA_CONTROL]

	MEND

	; ---------------------------------------------------------------------
	; SETUPMMU
	; -------
	; Setup the memory map world for the target. This macro is called by
	; INITMMU from ANGEL if required; or by uHAL when a standalone
	; program uses the mmu.
	
	; Level 1 entries always address 1MB each. The CPU takes the MB
	;	portion of the required address and uses it as an offset
	;	into the table. If the entry has bit 0 == 1, it looks in
	;	the level 2 table.
	; Level 2 entries always address 4KB each. If using 64KB level 2
	;	entries, each page table entry must be duplicated 16 times
	;	(in consecutive memory locations) in the page table.


	MACRO
$label  SETUPMMU	$tmp1, $tmp2, $tmp3, $tmp4, $tmp5
	ROUT

	; Setup the StrongARM MMU with a simple flat page table:
	; write the translation base register
	; set the domain register: domain0 CLIENT, domain1-15 NO_ACCESS
	; set up the page tables - all for domain0, flat address map
	;	$tmp2 x 1MB sections for DRAM
	;		- Read/Write all, cachable & bufferable
	;	PCI_DRAMSIZE x 1MB sections for DRAM in PCI space
	;		- R/W all, noncachable or bufferable
	;	EPROM_SIZE x 1MB sections for FLASH
	;		- R/W spvr, Read Only user, cachable, bufferable
	;	no access for aliased DRAM, and FLASH
	;	IO map , 21285 CSRs, PCI Config & PCI address space
	;		- noncachable, nonbufferable incl. aliases

	; Flush ITB's and DTB's
	; Flush the Icache

	MOV	$tmp1, #4		; 100, R--
	SET_LEDS	$tmp1, $tmp3

	LDR	$tmp1, =Level1tab	;TTB address is 2**14 aligned
	WRCP15_TTBase 	 $tmp1		;Initialise Translation Table Base reg.
	LDR	$tmp1, =1
	WRCP15_DAControl $tmp1		;Initialise Domain Access Control

	; First clear all TT entries - FAULT
	;
	LDR	$tmp4, =Level1tab
	LDR	$tmp3, =0
	LDR	$tmp1, =L1_TABLE_ENTRIES + L2_TABLE_ENTRIES
1
	STR	$tmp3, [$tmp4], #4
	SUBS	$tmp1, $tmp1, #1	; decrement loop count
	BNE	%B01

	; ----------------------------------------------------------------------
	; Configure DRAM section accesses
	; ----------------------------------------------------------------------
	;
	ADD	$tmp2, $tmp2, #256	; Make sure we cross 1MB boundary
	SUB	$tmp2, $tmp2, #DRAM_BASE ; Memory size = Top - Bottom

	MOV	$tmp3, $tmp2, LSR #20	; $tmp3 is now the no. of 1MB segments
	LDR	$tmp2, =Level1tab
	; DRAM_ACCESS determines access permissions
	LDR	$tmp1, =DRAM_BASE + DRAM_ACCESS
2
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B02

	; ----------------------------------------------------------------------
	; Now do PCI space - !!PCI space must follow DRAM space!!!
	; ----------------------------------------------------------------------
	; tmp3 - loop count
	; tmp1 - table entry
	;
	LDR $tmp3, =PCI_DRAMSIZE:SHR:20	; num of segments
	LDR $tmp1, =PCI_DRAMBASE + IO_ACCESS	; Access permisions and address 
20
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE %B20


	; ----------------------------------------------------------------------
	; Configure DRAM arrays + X bus access
	; ----------------------------------------------------------------------
	; 1MByte (slightly large but easier).
	;
	LDR	$tmp2, =Level1tab + DC21285_DRAM_A0MR:SHR:(20-2)
	; IO_ACCESS determines access permissions
	LDR	$tmp1, =DC21285_DRAM_A0MR + IO_ACCESS
	STR	$tmp1, [$tmp2], #4	; store TT entry

	; ----------------------------------------------------------------------
	; Configure ROM/Flash section accesses
	; ----------------------------------------------------------------------
	;
	; We use a L2 page table for ROM/Flash access. This code assumes the
	; ROM/Flash starts on a 1MB boundary.
	;
	LDR	$tmp5, =Level1tab + EPROM_BASE:SHR:(20-2)
	LDR	$tmp4, =Level2tab_ROM

	; Configure ROM large page accesses - 16 aliases per entry
	;
	LDR	$tmp2, =EPROM_SIZE	; Figure out out much EPROM..

	; Note: these two operations could be combined, but this makes how a
	; level 2 TLB works clearer.
	MOV	$tmp3, $tmp2, LSR #16	; $tmp3 is now the no. of 64KB segments
	MOV	$tmp3, $tmp3, LSL #4	; $tmp3 is now the no. of 4KB aliases
	; EPROM_ACCESS determines access permissions
	LDR	$tmp1, =(EPROM_BASE + EPROM_ACCESS)
3
	ADD	$tmp2, $tmp4, #L2_CONTROL
	STR	$tmp2, [$tmp5], #4

	; Add each entry aliased 16 times (so $tmp3 & 0xF == 0)
4	STR	$tmp1, [$tmp4], #4	; store large L2 TT entry
	SUB	$tmp3, $tmp3, #1	; decrement page count
	TST	$tmp3, #0xF
	BNE	%B04			; upto 16 L2 entries

	; Add 16 L2 entries per L1 entry  (1MB/64KB, so $tmp3 & 0xF0 == 0)
	ADD	$tmp1, $tmp1, #SZ_64K	; next page field
	TST	$tmp3, #0xF0
	BNE	%B04			; upto 16 L2 entries

	CMP	$tmp3, #0		; All done?
	BNE	%B03			; No, do another L1 entry.

	; ----------------------------------------------------------------------
	; Configure ARM CSR section accesses
	; ----------------------------------------------------------------------
	; 1MByte
	;
	LDR	$tmp2, =Level1tab + CSR_BASE:SHR:(20-2)
	; IO_ACCESS determines access permissions
	LDR	$tmp1, =CSR_BASE + IO_ACCESS
	STR	$tmp1, [$tmp2], #4	; store TT entry

	; ----------------------------------------------------------------------
	; Configure Outbound Write Flush
	; ----------------------------------------------------------------------
	; 16MB
	;
	LDR	$tmp2, =Level1tab + DC21285_OUTBOUND_WRITE_FLUSH:SHR:(20-2)
	LDR	$tmp3, =16
	LDR	$tmp1, =DC21285_OUTBOUND_WRITE_FLUSH + IO_ACCESS
5
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B05

	; ----------------------------------------------------------------------
	; Configure PCI IACK/special space
	; ----------------------------------------------------------------------
	; 16MB
	;
	LDR	$tmp2, =Level1tab + DC21285_PCI_IACK:SHR:(20-2)
	LDR	$tmp3, =16
	LDR	$tmp1, =DC21285_PCI_IACK + IO_ACCESS
6
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B06

	; ----------------------------------------------------------------------
	; Configure PCI type 1 configuration
	; ----------------------------------------------------------------------
	; 16MB
	;
	LDR	$tmp2, =Level1tab + DC21285_PCI_TYPE_1_CONFIG:SHR:(20-2)
	LDR	$tmp3, =16
	LDR	$tmp1, =DC21285_PCI_TYPE_1_CONFIG + IO_ACCESS
7
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B07

	; ----------------------------------------------------------------------
	; Configure PCI type 0 configuration
	; ----------------------------------------------------------------------
	; 16MB
	;
	LDR	$tmp2, =Level1tab + DC21285_PCI_TYPE_0_CONFIG:SHR:(20-2)
	LDR	$tmp3, =16
	LDR	$tmp1, =DC21285_PCI_TYPE_0_CONFIG + IO_ACCESS
8
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B08

	; ----------------------------------------------------------------------
	; Configure PCI I/O space
	; ----------------------------------------------------------------------
	; 16MB
	;
	LDR	$tmp2, =Level1tab + DC21285_PCI_IO:SHR:(20-2)
	LDR	$tmp3, =16
	LDR	$tmp1, =DC21285_PCI_IO + IO_ACCESS
9
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B09

	; ----------------------------------------------------------------------
	; Configure PCI Memory Space
	; ----------------------------------------------------------------------
	; 2GB
	;
	LDR	$tmp2, =Level1tab + DC21285_PCI_MEMORY:SHR:(20-2)
	LDR	$tmp3, =2048
	LDR	$tmp1, =DC21285_PCI_MEMORY + IO_ACCESS
10
	STR	$tmp1, [$tmp2], #4	; store TT entry
	ADD	$tmp1, $tmp1, #SZ_1M	; add section number field
	SUBS	$tmp3, $tmp3, #1	; decrement loop count
	BNE	%B10

	; ----------------------------------------------------------------------
	; Configure DCache Clean Virtual Addressing
	; ----------------------------------------------------------------------
	; Map a high virtual address to EPROM with Read-Only user space
	; Access is domain0, ap=10 (SVR r/w, other r/o), C=1, B=0). This
	; range of addresses is used to flush out the D-Cache.
	; NOTE: Assumes DCache is less than 1Mb :-)
	;
	LDR	$tmp2, =Level1tab + CLEAN_BASE:SHR:(20-2)
	LDR	$tmp1, =DC21285_SA_CACHE_FLUSH + DRAM_ACCESS
	STR	$tmp1, [$tmp2]

	; ----------------------------------------------------------------------
	; Flush everything and turn on the MMU
	; ----------------------------------------------------------------------
	WRCP15_FlushITB_DTB $tmp1	; Flush ITB's + DTB's
	WRCP15_FlushIC_DC	$tmp1	; Flush the ICache and DCaches
	NOP				; make sure that the pipe is empty
	NOP
	NOP

	MOV	$tmp1, #5		; 101, R-Y
	SET_LEDS	$tmp1, $tmp3

	MEND

	; ----------------------------------------------------
	;
	; ANGEL and uHAL INITMMU macro, calls the core SETUPMMU macro if
	; the MMU is enabled and then sets ICache, DCache & WBuffer as
	; required.  On certain system where the target does not have a MMU,
	; then this  macro is almost (but not quite) a NOP: It is good to know
	; the MMU/cache register has been written and just using the ICache &
	; WBuffer on their own is still a useful gain.
	;
	; ----------------------------------------------------
	MACRO
$label  INITMMU	 $tmp1, $tmp2, $tmp3, $tmp4, $tmp5, $tmp6

EBSAInitMMU  

 IF :LNOT: :DEF: ENABLE_MMU

ENABLE_MMU	EQU	1		; Angel flags not defined in uHAL
CACHE_SUPPORTED	EQU	1

 ENDIF

 IF ENABLE_MMU = 1

	;LDR	$tmp2, =uHAL_MEMORY_SIZE	; Pass the top of memory
	LDR 	$tmp2, =PCI_DRAMBASE		; Pass top of non PCI memory
	SETUPMMU	$tmp1, $tmp2,$tmp3, $tmp4, $tmp5

	; Enable MMU, alignment faults and IC/DC/WB as required
	; 
	LDR	$tmp2, =EnableMMU	; NOTE:  no alignment checks enabled 

 ELSE

	LDR	$tmp2, =0		; No MMU

 ENDIF	; ENABLE_MMU

 IF CACHE_SUPPORTED = 1

    IF ENABLE_MMU = 1

	LDR	$tmp1, =(IC_ON + DC_ON + WB_ON)

    ELSE

	; Cannot use Data Cache without MMU
	LDR	$tmp1, =(IC_ON + WB_ON)

    ENDIF

 ELSE

	; Handy performance boost to be had even if no MMU is available
	;LDR	$tmp1, =(IC_ON + WB_ON)
	LDR	$tmp1, =0

 ENDIF	; CACHE_SUPPORTED

	ORR	$tmp1, $tmp1, $tmp2

 IF {ENDIAN} = "big"

	ORR	$tmp1, $tmp1, #EnableBigEndian

 ENDIF	; BIG_ENDIAN

	WRCP15_Control	$tmp1	; Update control register
	NOP
	NOP
	NOP
	NOP

	MOV	$tmp1, #6			; 110, RG-
	SET_LEDS	$tmp1, $tmp2

	MEND

	; ---------------------------------------------------------------------
	; INITTIMER
	; ---------
	; This macro is provided purely as a holder for any code that
	; may be required to initialise a hardware timer as part of
	; the reset sequence. Angel does *NOT* make use of any timer
	; resources, so by default this macro should not be required
	; to do anything. However, under exceptional circumstances
	; special code may be required to ensure that Angel starts
	; cleanly and that later application specific code can have
	; full control of the timer.
	;
	MACRO
$label  INITTIMER	$w1, $w2
	; Last access to LEDs is here, turn them all off..
	MOV	$w1, #0
	SET_LEDS	$w1, $w2
	MEND

	; ---------------------------------------------------------------------
	; REVERSE
	; -------
	; Quick word byte-order reversal macro. Handy for endian stuff.
	MACRO
$label  REVERSE	$w1, $w2		; w1:	 D    C    B    A
	EOR	$w2, $w1, $w1, ROR #16	; w2:	D^B, C^A, B^D, A^C
	BIC	$w2, $w2, #0xff0000	; w2:	D^B,  0 , B^D, A^C
	MOV	$w1, $w1, ROR #8	; w1:	 A    D    C    B
	EOR	$w1, $w1, $w2, LSR #8	; w2>>8: 0   D^B   0   B^D
	MEND				; w1:	 A    B    C    D

	; ---------------------------------------------------------------------
	; IOBADDR
	; -------
	; Macro to calculate the memory address corresponding to an I/O port
	; for byte I/O. This relies on the fact that all the registers are
	; mapped into the low-order byte of seperate dwords.
	MACRO
$label	IOBADDR $reg, $io_port

 IF {ENDIAN} = "big"
	LDR	$reg, =((($io_port) :SHL: IOShift) + XBUS_DEV_1 + 3)
 ELSE
	LDR	$reg, =((($io_port) :SHL: IOShift) + XBUS_DEV_1)
 ENDIF
	MEND

	; ---------------------------------------------------------------------
	; GETSOURCE
	; ---------
	; This macro is used to read the current interrupt source
	; activity status for the Angel device driver interrupts:

	MACRO
$label  GETSOURCE	$w1, $w2
	;; Find out what caused the interrupt

 IF :DEF: AMBA_SERIAL
	; FIXME: 21285 serial port To Be Done
	IOBADDR $w1, (COM1Port+InterruptId)	; Get interrupt id (from 16550)
	MOV	$w1, #IH_SERIAL
 ELSE
	MOV	$w1, #IH_SERIAL
 ENDIF
	MEND

	; ---------------------------------------------------------------------
	; READ_INT
	; --------
	; uHAL macro to read which interrupt(s) is active (result in $w1)

	MACRO
$label  READ_INT	$w1, $w2, $w3

	LDR	$w2, =DC21285_IRQStatus
	LDR	$w1, [$w2]

	MEND

	; ---------------------------------------------------------------------
	; CACHE_IBR
	; ---------
	; This macro implements an instruction barrier for a range of addresses
	; (i.e. it makes instruction and data memory coherent for this range) 
	; w1 contains the start of the range , w2 the next address after the end
	; Note that w1 will be corrupted

	MACRO
$label  CACHE_IBR	$w1, $w2, $temp, $temp2

	ASSERT  ( $w1 = r0 )

	; Protect lr from the SWI
	STMFD	sp!, {lr}
 
	; Save the old mode
	MRS	$temp, CPSR
	AND	$temp, $temp, #ModeMask
	CMP	$temp, #USRmode
	BNE	NonUSRmodeCase1

	; If in USR mode then get into SVC mode
	MOV	$temp2, r0
	LDR	r0, = angel_SWIreason_EnterSVC
	SWI	angel_SWI_ARM
	MOV	r0, $temp2
	
NonUSRmodeCase1
	; Clean the Dcache entries
01	
	WRCP15_CleanDCentry $w1
	ADD	$w1, $w1, #32
	CMP	$w1, $w2
	BLT	%B01

	; Drain the write buffer
	WRCP15_DrainWriteBuffer $w1	

	; Flush the Icache
 	MOV	$w1, #0
 	WRCP15_FlushIC $w1

	; Clear the pipeline..
	NOP
	NOP
	NOP
	NOP
	
	; Return to old mode if we changed in the first place
 	CMP	$temp, #USRmode
 	BNE	NonUSRmodeCase2
	; If originally in USR mode then leave SVC mode properly
 	IMPORT  Angel_ExitToUSR
 	BL	Angel_ExitToUSR
	
NonUSRmodeCase2
	; Restore lr
	LDMFD	sp!, {lr}
	MEND


;===================================
; This routine initialises the X-Bus

	MACRO
$label  INIT_XBUS	$w1,$w2

	ldr	$w1, =SA_CTL_INIT
	ldr	$w2, =CSR_BASE
	str	$w1, [$w2, #SA_CONTROL]

	ldr	$w1, =INIT_XBUS_CYCLE
	str	$w1, [$w2, #XBUS_CYCLE]

;
; set strobe mask = 0xFC for all devices
;
	ldr	$w1, =INIT_XBUS_STROBE
	str	$w1, [$w2, #XBUS_STROBE]

;
; Write zeros to the output of LPT2 
; [Sean:	 stops a power drain ]
;
	ldr	$w1, =0x40011DE0
	ldr	$w2, =0
	str	$w2, [$w1]
	
	MEND

	MACRO
$label	DISABLE_INTS	$w1, $w2

	MOV	$w1, #7
	SET_LEDS	$w1, $w2

	LDR	$w2, =CSR_BASE
	; All interrupts should be off by default - this makes sure
	LDR	$w1, =0xFFFFFFFF
	STR	$w1, [$w2, #IRQ_ENABLE_CLEAR]
	STR	$w1, [$w2, #FIQ_ENABLE_CLEAR]

	; Explicitly disable the UARTs (COM1 and COM2) from interrupting
	; This code relies on the fact that all the UART registers are
	; mapped into seperate dwords.
	IOBADDR	$w1, COM1Port
	LDR	$w2, =0xF
	STRB	$w2, [$w1, #CntlModem]
	LDR	$w2, =0                 ; disable interrupt
	STRB	$w2, [$w1, #EnableInt]
	IOBADDR	$w1, COM2Port
	LDR	$w2, =0xF
	STRB	$w2, [$w1, #CntlModem]
	LDR	$w2, =0                 ; disable interrupt
	STRB	$w2, [$w1, #EnableInt]

	MEND

	; ---------------------------------------------------------------------
	; uHAL macro to initialise external interrupts when running standalone

	MACRO
$label 	INIT_INTS	$w1, $w2, $w3

 IF :DEF: STANDALONE

	INIT_XBUS	$w1, $w2

	DISABLE_INTS	$w1, $w2

 ENDIF

	MEND
	; ---------------------------------------------------------------------

	; ---------------------------------------------------------------------
	; Macro used for debugging without having target-specific code in uHAL

	MACRO
$label 	DO_DEBUG	$w1, $w2, $w3

	SET_LEDS	$w1, $w2	; Simple LED lighting on EBSA-285

	MEND

	; ---------------------------------------------------------------------
	; macro to set the LEDs to the value given in $w1
	; NOTE: this should be the same as SetLEDs in driver.s

	MACRO
$label 	SET_LEDS	$w1, $w2

	EOR	$w1, $w1, #ALL_LEDS	; Invert value (zero turns LED on)
	LDR	$w2, =LED_BANK
	STRB	$w1, [$w2]

	MEND
	; ---------------------------------------------------------------------

;;; ==============================
;;; Primary boot loader for StrongARM.
;;; ==============================

;;; This file is the source of the primary (Flash) boot loader for StrongARM.
;;; The boot loader:
;;; 
;;;	Reads the image selection rotary switch
;;;
;;;	If the selected image is 0 the system will perform the powerup
;;;	diagnostics[tba] and then enter the Angel debug agent.
;;; 
;;;	If the jumpers do not select image 0:
;;;
;;;		Search for the image in Flash, and verify that the checksum
;;;		is correct.
;;;
;;;		If the image is in executable AIF format jump to the image.
;;;
;;;		Otherwise load the image into memory at the addresses defined
;;;		in the AIF header and jump to the image's entry point.
;;;
;;;	If the selected image is not found in Flash, or is corrupt (bad
;;;	checksum), behave as though the selected image is image 0.
;;;
;;; This code assumes that Angel has already initialised the systems, so
;;; does not switch memory maps.
;;;
;;; This file is not a stand alone assembly, instead it is included as an
;;; include file in the startup code for a special version of Angel.
		

	MACRO
	BOOTLD	$w1, $w2

	LDR	$w1, =IMAGE_SWITCH	; Get the switch setting
	LDRB	$w1, [$w1]
	AND	$w1, $w1, #0x0f
	EORS	$w1, $w1, #0x0f		; Invert
	BEQ	BootFlashZero		; If zero continue with this block.

	;; Find the image; we have to keep track of which blocks are allocated
	;; so we might as well start at block 0;
	MOV	r1, #0			; Number of current flash block
	MOV	r2, #FLASH_BASE		; Address of current flash block
	MOV	r3, #0			; Allocated block bit map
	MOV	r9, #1			; Useful for bit tests
	LDR	r5, =0x00aa5500		; Template for signature check

Flash_Loop
	CMP	r1, #(((FLASH_END - FLASH_BASE)/FLASH_BLOCK_SIZE) + 1)
	BEQ	BootFlashZero		; Image not found

	;; Check if this can be valid flash block
	TST	r9, r3,  LSL r1
	BNE	Next_Flash_Block 	; Block allocated in earlier image

	LDR	r6, [r2, #FH_NUMBER]	; Load the block number and signiture
	BIC	r7, r6, #0xFF		; Clear the block number
	CMP	r7, r5			; Check the signiture
	BNE	Next_Flash_Block	; Block not programmed

	;; Block contains an image header, check if is the one we want
	AND	r6, r6, #0xFF
	CMP	r0, r6		
	BEQ	Image_Found		; Found It !

	;; Wrong image, update the allocation mask and continue
	LDR	r6, [r2, #FH_MAP]
	ORR	r3, r3, r6		; Add in the new bits

Next_Flash_Block
	ADD	r1, r1, #1
	ADD	r2, r2, #FLASH_BLOCK_SIZE
	B	Flash_Loop

;;; We have found the image, check it and then boot

Image_Found
	;; Checksum check. The checksum, which should be 0, is simply the
	;; sum of the 32 bit words of the image (including the header.  Extra
	;; bytes in the last word are treated as 0xff, since this is the
	;; value that will be left in unprogrammed flash bytes.
	LDR	r3, [r2, #FH_MAP]	; Map
	LDR	r0, [r2, #FH_LENGTH]	; length of executable
	MOV	r9, r2			; First block
	ADD	r5, r2, #FLASH_BLOCK_SIZE
	MOV	r6, #1
	MOV	r7, #0			; Checksum so far
	MOV	r8, r6, LSL r1		; Convert r1 into a bit to test mask
Block_check_loop
	TST	r3, r8
	BEQ	Check_Next_Block	; This block is not in the map
Word_check_loop
	CMP	r9, r5
	BEQ	Check_Next_Block	; Block finished
	LDR	r6, [r9], #4
	ADD	r7, r7, r6
	SUBS	r0, r0, #4
	BLE	Checksum_calculated
	B	Word_check_loop
Check_Next_Block
	BICS	r3, r3, r1
	BEQ	BootFlashZero		; We never reached the end of the
					; image, something is corrupt

	MOV	r8, r8, LSL #1
	MOV	r9, r5
	ADD	r5, r5, #FLASH_BLOCK_SIZE
	B	Block_check_loop

Checksum_calculated
	CMP	r7, #0			; Check the checksum
	BNE	BootFlashZero

	LDR	r0, [r2, #FH_TYPE]
	LDR	r3, =0xE1A00000		; NOOP
	CMP	r0, r3			; Check for executable from FLASH
	BEQ	NonExec

	LDR	r0, [r2, #FH_BOOTFLAGS] 

	TST	r0, #FH_NOBOOTMASK	; If noboot mask is set don't boot it
	BNE	BootFlashZero

	MOV	pc, r2			; If it is an executable image simply
					; jump to it. This will jump to the
					; high memory copy.

NonExec
	;; We have a nonexecutable AIF, so we have to copy it into memory.
	;; Assumes memory switched and DRAM to be initialised. Note that we
	;; are executing from the high memory copy of the image, and r2
	;; contains the address of the high memory copy of the image to be
	;; loaded. r1 contains the block number.
	MOV	r10, r1			; Protect registers from INIT_RAM
	MOV	r11, r2
	INIT_RAM	r0, r1, r5
	MOV	r2, r11			; Done, now restore hi mem address
	MOV	r1, r10			; and block no.
	
	;; Now load the image.  Use the map to decide which blocks to load.
	LDR	r3, [r2, #FH_MAP]	; Map
	LDR	r0, [r2, #FH_AIF_BASE]	; Destination of load
	ADD	r9, r2, #FH_END		; Source of load
	ADD	r5, r2, #FLASH_BLOCK_SIZE
	MOV	r6, #1
	MOV	r1, r6, LSL r1		; Convert r1 into a bit, test the mask
Block_load_loop
	TST	r3, r1
	BEQ	Load_Next_Block		; This block is not in the map
Word_load_loop
	CMP	r9, r5
	BEQ	Load_Next_Block		; Block finished
	LDR	r6, [r9], #4
	STR	r6, [r0], #4
	B	Word_load_loop
Load_Next_Block
	BICS	r3, r3, r1
	BEQ	Load_Done
	MOV	r1, r1, LSL #1
	MOV	r9, r5
	ADD	r5, r5, #FLASH_BLOCK_SIZE
	B	Block_load_loop
Load_Done
	;; The load is complete, find the entry point and jump to it
	LDR	r0, [r2, #FH_BOOTFLAGS] 
	TST	r0, #FH_NOBOOTMASK	; If noboot mask is set don't boot it
	BNE	BootFlashZero
	LDR	r0, [r2, #FH_AIF_BASE]	; The entry point is the base address
	LDR	r1, [r2, #FH_AIF_ENTRY]	; plus the entry point offset
	ADD	r1, r0, r1
	MOV	pc, r1			; Jump into the code!

BootFlashZero
	;; Continue with Angel startup, in containing file.

	MEND

	; ---------------------------------------------------------------------

	ENDIF	 ; ebsa285_s

 IF :DEF: OPT
	OPT	(old_opt)	; restore previous listing options
 ENDIF

	; ---------------------------------------------------------------------
	END	; EOF target.s

