/***********************************************************************
 * Copyright  Intel Corporation 1998.  All rights reserved.
 ***********************************************************************

        Timer functions

 ***********************************************************************/

#include "devconf.h"

#if ETHERNET_SUPPORTED

#include "timer.h"


/*
 * 21285 Timer register definitions.
 *
 * 4 timers, each has four registers (Load, Value, Control and Clear).
 */

typedef struct TimerStruct {
  unsigned int TimerLoad ;
  unsigned int TimerValue ;
  unsigned int TimerControl ;
  unsigned int TimerClear ;
  unsigned int dummy[4] ;
} TimerStruct_t;

volatile TimerStruct_t *TimerBase = (TimerStruct_t *)(DC21285_ARMCSR_BASE+TIMER_3_LOAD) ;

/* This routine stops the specified timer hardware. */
void stop_timer(void)
{
  unsigned int mode ;
  volatile TimerStruct_t *Timer = TimerBase;
                                                     
  /* clear the interrupt (if any) */
  Timer->TimerClear = 1 ;

  /* turn the timer off (do not disturb the other mode bits) */
  mode = Timer->TimerControl ;
  mode &= ~TIMER_CONTROL_ENABLE ;
  Timer->TimerControl = mode ; 
}


/* 
        This routine starts the specified timer hardware. 
        3.68 MHz
*/
void start_timer(U32 counter)
{
  unsigned int mode ;
  volatile TimerStruct_t *Timer = TimerBase;

  /* clear the interrupt (if any) */
  Timer->TimerClear = 1 ;



  /* turn the timer on */
  mode = Timer->TimerControl ;
  mode |= TIMER_CONTROL_ENABLE ;
  mode |= 0x0cc;
  Timer->TimerControl = mode ; 
  
  /* set its count */
  Timer->TimerLoad = counter;   
}

int timer_expired(void)
{
        if ( (volatile)(*(U32*)(DC21285_ARMCSR_BASE + IRQ_RAW_STATUS)) & TIMER3_INT )
                return TRUE;
        else 
                return FALSE;   
}


/*
    Uses 21285 timer 3 
        The clock is around 3.68 MHz so if we wait 4 ticks per uS we'll
        be in the right area.
*/
void _usleep(U16 period)
{
U32 ticks;

        ticks = period * 4;     
        start_timer(period); 
                while (FALSE == timer_expired());
        
stop_timer();   
}

void _msleep(U16 tick)                                   
{
U32 i=0;
                
        for(i = 0; i < tick; i++) {   
        	start_timer(mSEC_1);
                while (FALSE == timer_expired());
        }
stop_timer();
}

                                                              

/*
 use a timer    sleep.
*/
void _sleep( U16 period)
{
        U16 i;

        for (i = 0; i < period; i++) {
                _msleep(1000);  
        }       
}


#endif

