/****************************************************************************
**                                                                          *
**  Copyright  1997						     								*
**  by Digital Equipment Corporation, Maynard, Massachusetts.		     	*
**                                                                          *
**  All Rights Reserved							     						*
**                                                                          *
**  Permission  is  hereby  granted  to  use, copy, modify and distribute   *
**  this  software  and  its  documentation,  in  both  source  code  and   *
**  object  code  form,  and without fee, for the purpose of distribution   *
**  of this software  or  modifications  of this software within products   *
**  incorporating  an  integrated   circuit  implementing  Digital's  		*
**  StrongARM architecture,  regardless  of the  source of such integrated 	*
**	circuit, provided that the  above copyright  notice and this permission *
**  notice appear  in  all copies,  and  that  the  name  of  Digital  		*
**  Equipment Corporation  not  be  used  in advertising or publicity 		*
**  pertaining to distribution of the  document  or  software without 		*
**  specific, written prior permission.							            *
**                                                                          *
**  Digital  Equipment  Corporation   disclaims  all   warranties  and/or   *
**  guarantees  with  regard  to  this  software,  including  all implied   *
**  warranties of fitness for  a  particular purpose and merchantability,   *
**  and makes  no  representations  regarding  the use of, or the results   *
**  of the use of, the software and documentation in terms of correctness,  *
**  accuracy,  reliability,  currentness  or  otherwise;  and you rely on   *
**  the software, documentation and results solely at your own risk.	    *
**                                                                          *
**  StrongARM is a trademark of Digital Equipment Corporation               *
**                                                                          *
*****************************************************************************/

#ifndef LINT
extern char *rcsid;
#endif



#include "devconf.h"
#if (ETHERNET_SUPPORTED == 1)

#include "stddef.h"
#include "system.h"
#include "platform.h"
#include "net_buff.h"
#include "angel.h"
#include "debugio.h"


#define PCI_MEMORY PCI_MEM


/*
	These buffers are for the socket layer
	The device driver will now use these - PCI can address
	the same space as the SA
*/


static inuse_socket_table *ist_buffers;
static char *frame_buffers = NULL;
static packet_queue *pq_buffers;

static char ist_q[NUM_OF_IST_BUFS];
static char frame_q[NUM_OF_FRAME_BUFS];
static char pq_q[NUM_OF_PQ_BUFS];
static void* netBufferAllocBase = NULL;


void	socket_buffer_init(void)
{
	unsigned int i;
	
	if (NULL != frame_buffers)	 
		return;
		
	i = (unsigned int)EthernetBufferBase;

	
	frame_buffers = (char*)EthernetBufferBase;
			
	ist_buffers = (inuse_socket_table *)EthernetQBufferBase ; // needs the 4 for alignment

	pq_buffers = (packet_queue*)(ist_buffers + 
			(NUM_OF_IST_BUFS * sizeof(inuse_socket_table)));
	
	netBufferAllocBase = (void*)(frame_buffers +
		 ((SRING_SIZE + RRING_SIZE + 2) * ETHER_BUFFER_SIZE * 2)) ; // 2 setup frames - 1 for breathing space


	// Trouble - if not enough room
	if  ( Ethernet_Table_Size < (( sizeof(inuse_socket_table) * NUM_OF_IST_BUFS) 
			+ (NUM_OF_PQ_BUFS * sizeof(packet_queue)) )) {
			
		while (1);
	}



	for (i=0;i<NUM_OF_IST_BUFS;i++) {
		ist_q[i] = FALSE;
	}
	for (i=0;i<NUM_OF_PQ_BUFS;i++) {
		pq_q[i] = FALSE;
	}
	for (i=0;i<NUM_OF_FRAME_BUFS;i++) {
		frame_q[i] = FALSE;
	}
}


/*
	Allocates word aligned areas ie bits:10 = 00
*/
void * net_buffer_alloc(U32 size)
{
	void *base;
	if (netBufferAllocBase == NULL)
		socket_buffer_init();
		
// FIXME FIXME Should check size fits in mem allocation here - bodging for now
	if ((int)netBufferAllocBase % 4)
		netBufferAllocBase = (void *) (((unsigned int)netBufferAllocBase + 4) & 0xfffffffc);
	base = netBufferAllocBase;
	
	netBufferAllocBase = (void*)(((U32)netBufferAllocBase) + size);

	return base;
}

inuse_socket_table* istmalloc(size_t size) /* I know how big it is really, just checking */
{
	int i;

	if (size != sizeof(inuse_socket_table))	{
		return NULL;
	}
		
	for(i=0;i<NUM_OF_IST_BUFS;i++) {
		if (ist_q[i] == FALSE) {
			ist_q[i] = TRUE;
			return &ist_buffers[i];
		}
	}
		return NULL;		
}

unsigned char* framemalloc(size_t size)
{
	int i;
	
	if (size != ETHER_BUFFER_SIZE) {
		return NULL;
	}

	for(i=0;i<NUM_OF_FRAME_BUFS;i++) {
		if (frame_q[i] == FALSE) {
			frame_q[i] = TRUE;

			return (unsigned char*)(frame_buffers + (i * ETHER_BUFFER_SIZE));
		}
	}
	return NULL;		
}
	


packet_queue* pqmalloc(size_t size)
{
	int i;

	if (size != sizeof(packet_queue)) {
		return NULL;
	}
	
	for(i=0;i<NUM_OF_PQ_BUFS;i++) {
		if (pq_q[i] == FALSE) {
			pq_q[i] = TRUE;
			return &pq_buffers[i];
		}
	}
	return NULL;		
}


void istfree(inuse_socket_table* ptr)
{
	int i;
	
	for(i=0;i<NUM_OF_IST_BUFS;i++) {
	 	if (&ist_buffers[i] == ptr) { /* One of mine, free it */
			ist_q[i]=FALSE;
			return;
	 	}
	
	}
	return ; /* Not one of mine */	
}

bool framefree(unsigned char *ptr)
{
	int i;

	
	for(i=0;i<NUM_OF_FRAME_BUFS;i++) {
	 	if ((unsigned char*)(frame_buffers + (i * ETHER_BUFFER_SIZE)) == ptr) { /* One of mine, free it */
			frame_q[i]=FALSE;
			
			return FALSE;
	 	}
	
	}
	return TRUE; /* Not one of mine */
}

void pqfree(packet_queue* ptr)
{
	int i;
	

	for(i=0;i<NUM_OF_PQ_BUFS;i++) {
	 	if (&pq_buffers[i] == ptr) { /* One of mine, free it */
			pq_q[i]=FALSE;
			return;
	 	}
	
	}
	return ; /* Not one of mine */
}



#endif
