#ifndef __NET_BUFFER_H_LOADED
#define __NET_BUFFER_H_LOADED
/****************************************************************************
**                                                                          *
**  Copyright  1997						     								*
**  by Digital Equipment Corporation, Maynard, Massachusetts.		     	*
**                                                                          *
**  All Rights Reserved							     						*
**                                                                          *
**  Permission  is  hereby  granted  to  use, copy, modify and distribute   *
**  this  software  and  its  documentation,  in  both  source  code  and   *
**  object  code  form,  and without fee, for the purpose of distribution   *
**  of this software  or  modifications  of this software within products   *
**  incorporating  an  integrated   circuit  implementing  Digital's  		*
**  StrongARM architecture,  regardless  of the  source of such integrated 	*
**	circuit, provided that the  above copyright  notice and this permission *
**  notice appear  in  all copies,  and  that  the  name  of  Digital  		*
**  Equipment Corporation  not  be  used  in advertising or publicity 		*
**  pertaining to distribution of the  document  or  software without 		*
**  specific, written prior permission.							            *
**                                                                          *
**  Digital  Equipment  Corporation   disclaims  all   warranties  and/or   *
**  guarantees  with  regard  to  this  software,  including  all implied   *
**  warranties of fitness for  a  particular purpose and merchantability,   *
**  and makes  no  representations  regarding  the use of, or the results   *
**  of the use of, the software and documentation in terms of correctness,  *
**  accuracy,  reliability,  currentness  or  otherwise;  and you rely on   *
**  the software, documentation and results solely at your own risk.	    *
**                                                                          *
**  StrongARM is a trademark of Digital Equipment Corporation               *
**                                                                          *
*****************************************************************************/

/* 
 * $Id: net_buff.h,v 2.3 1997/08/27 10:49:27 smithg Exp $
 */

/*
 * MODULE DESCRIPTION:
 * 
 *     Buffer allocator for ethernet shared memory on EBSA110. This buffer allocator
 *     does not allow buffers to be freed.  Because the memory is only 16 bits wide
 *     (on a 32 bit bus) the buffers should only be accessed through the macros in 
 *     this file.
 *
 * HISTORY:
 * 
 * $Log: net_buff.h,v $
 * Revision 2.3  1997/08/27  10:49:27  smithg
 * added EthernetBufferBase
 *
 * Revision 2.2  1997/08/21 14:09:07  smithg
 * 21285 working 1
 *
 * Revision 2.1  1997/08/15 12:20:48  smithg
 * eb285
 *
 * Revision 2.0  1997/07/23 13:26:04  smithg
 * ebsa285 revisions
 *
 * Revision 1.2  1997/07/23 10:57:10  smithg
 * Header updated
 *
 * Revision 1.1  1997/07/14 14:45:49  smithg
 * Initial revision
 *
 * Revision 1.3  1997/07/04 10:03:34  smithg
 * changed to unix format from dos format
 *
 * Revision 1.2  1997/07/03 10:32:32  smithg
 * system.h includes removed
 *
 * Revision 1.1  1996/03/28 11:14:54  rusling
 * Initial revision
 *
 * Revision 1.2  1996/01/15  10:27:55  rusling
 * First working version.
 *
 * Revision 1.1  1996/01/09  14:26:48  berent
 * Initial revision
 *
 *
 *
 */

#include "socket.h"
#include "dec21040.h"
#include "angel.h"
#include "cdefs.h"
extern unsigned char * net_buffer_base;

/* net_buffer_address is the address of a buffer as seen by the ethernet
   controller. This is different from the address seen by the processor */
typedef unsigned short net_buffer_address;

/* net_buffer_init_module - initialise the module
 */
extern void net_buffer_init_module(void);

/* net_buffer_alloc - allocate a region of memory accessable by
 * network DMA devices
 *
 * Argument:
 *    size - number of bytes to allocate
 *
 * Result:
 *    Pointer to start of buffer
 */
extern void* net_buffer_alloc(U32 size);

/* net_mem_compare - compares a piece of shared memory with a piece of normal
 * memory. Arguments and function as for memcmp, except that second argument
 * must be a net_buffer_address.
 */
extern net_mem_compare(void const * const mem,const net_buffer_address buff, const int size);

/* Write a byte to the ethernet buffer space */
#define NET_BUFFER_WRITE_BYTE(address, data) \
{ \
    *(net_buffer_base + (int)((((address) & 0xfffe) << 1) | (address & 0x0001))) = data; \
}

/* Write a 2 byte quantity to the ethernet buffer space (address must be even) */
#define NET_BUFFER_WRITE_SHORT(address, data) \
{ \
    *(unsigned short *)(net_buffer_base + (int)(((address) & 0xfffe) << 1)) = data; \
}

#define NET_BUFFER_READ_BYTE(address) \
(*(net_buffer_base + (int)((((address) & 0xfffe) << 1) | ((address) & 0x0001)))) 

#define NET_BUFFER_READ_SHORT(address) \
(*(unsigned short *)(net_buffer_base + (int)(((address) & 0xfffe) << 1))) 


#define EthernetBufferBase 0xE80000 // @14.5 Meg, must be word aligned



#define  	NUM_OF_IST_BUFS		4 // Num of open sockets allowed
#define 	NUM_OF_PQ_BUFS 		8 // Length of waiting rx packets
#define		NUM_OF_FRAME_BUFS	((SRING_SIZE + RRING_SIZE + 1)*2)
#define 	NUM_OF_FRAME_BUFS_TO_ALLOC 	(1 + (SRING_SIZE + RRING_SIZE) * 2) // Number of frames in the system (rx and tx)


unsigned char* framemalloc(U32 size);
inuse_socket_table* istmalloc(U32 size);
packet_queue* pqmalloc(U32 size);
void istfree(inuse_socket_table* ptr);
void pqfree(packet_queue* ptr);
bool framefree(unsigned char *ptr);
void socket_buffer_init(void);


#endif /* __NET_BUFFER_H_LOADED */

