//
// Copyright (C) 1997 by
// Digital Equipment Corporation, Maynard, Massachusetts.
// All rights reserved.
//
// This software is furnished under a license and may be used and copied
// only  in  accordance  of  the  terms  of  such  license  and with the
// inclusion of the above copyright notice. This software or  any  other
// copies thereof may not be provided or otherwise made available to any
// other person.  No title to and  ownership of the  software is  hereby
// transferred.
//
// The information in this software is  subject to change without notice
// and  should  not  be  construed  as a commitment by Digital Equipment
// Corporation.
//
// Digital assumes no responsibility for the use  or  reliability of its
// software on equipment which is not supplied by Digital.
//
//
// Abstract:    
//
// Author:      
//
// Revision:    
//
// Modifications:
//
//     $History: $
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "platform.h"
#ifndef ANGEL
#include "cdef.h"
#endif
#include "fmu.h"
#ifndef ANGEL
#include "pciproto.h"
#endif

/*
 *  Global variables
 */
extern unsigned volatile char *flash_base ;
extern char blocks[NUMBER_OF_BLOCKS] ;
unsigned int flash_block_size = FLASH_BLOCK_SIZE ;

/*
 *  forward declarations.
 */
int check_flash_id(volatile void *flash) ;
unsigned int read_flash_dword(int offset) ;
int write_flash_dword(int offset, unsigned int data) ;
void delete_flash_block(int blockno) ;

/* ---------- F L A S H --- S U P P O R T --- R O U T I N E S ----------------*/

/* 
 * sync PCI transactions when NOT angel 
 */
static void sync(void)
{
#ifndef ANGEL
  volatile unsigned int x ;

  x = *(unsigned int *)flash_base ;
#endif 
}

/*
 *  Check to see if there is some flash at the location specified.
 */
int check_flash_id(volatile void *flash) 
{
    volatile unsigned int *flashPtr = (unsigned int *)flash ; 
    unsigned int man, id ;

    /* read the manufacturer's id */
    sync() ;
    *flashPtr = 0xffffffff ;        
    sync() ;
    *flashPtr = 0x90909090 ;        
    man = *flashPtr ;        

    /* correct manufacturer's id? */
    if (man != 0x89898989) {
       /* Make sure the flash (if it exists) is back in normal mode so 
          that we can do user I/O via firmware in flash */
      sync() ;
      *flashPtr = 0xffffffff;
      return FALSE ;
    } 

    /* read the device id */
    id = *(flashPtr + 1) ;

    /* correct device id? */
    if (id < 0xA1A1A1A1) {
       /* Make sure the flashPtr (if it exists) is back in normal mode so 
          that we can do user I/O via firmware in flash */
      sync() ;
      *flashPtr = 0xffffffff;
      return FALSE ; 
    }

    /* whoopee, we found it */

    /* Make sure the flashPtr is back in normal mode so 
       that we can do user I/O via firmware in flash */
    sync() ;
    *flashPtr = 0xffffffff;
    return TRUE ;
}

/* relies on flash_base to be correctly set */
unsigned int read_flash_dword(int offset)
{
  volatile unsigned int *flashLocation ; 

#ifndef ANGEL
  //
  // this code will notice that the offset into the rom 
  // needs to swapped...
  //
  if (offset <= 0x1F) {
    offset += 0x20;
  }
  else {
    if ((offset >= 0x20) && (offset < 0x40)) {
      offset -= 0x20;
    }
  }
#endif
  /* where are we reading from? */
  flashLocation = (unsigned int *)(flash_base + offset) ;

  /* align the address */
  flashLocation = (unsigned int *)((unsigned int)flashLocation & 0xFFFFFFFC) ;

  /* do the read */
  return *flashLocation ;
}

/* relies on flash_base to be correctly set */ int
write_flash_dword(int offset, unsigned int data) 
{ 
  volatile unsigned int *flashBasePtr = (unsigned int *)flash_base ; 
  volatile unsigned int *flashLocation ; 
  int status ;

#ifndef ANGEL
  //
  // this code will notice that the offset into the rom 
  // needs to swapped...
  //
  if (offset <= 0x1F) {
    offset += 0x20;
  }
  else {
    if ((offset >= 0x20) && (offset < 0x40)) {
      offset -= 0x20;
    }
  }
#endif

  /* where are we writing to? */
  flashLocation = (unsigned int *)(flash_base + offset) ;

  /* align the address */
  flashLocation = (unsigned int *)((unsigned int)flashLocation & 0xFFFFFFFC) ;

  /* set the device into write mode */
  sync() ;
  *flashLocation = 0x40404040 ;
  
  /* write the data */
  sync() ;
  *flashLocation = data ;

  /* successful? */
  do {
    sync() ;
    *flashLocation = 0x70707070 ;
    status = *flashLocation ;
  } while ((status & 0x80808080) != 0x80808080) ;
  
  /* Make sure the flash is back in normal mode so 
     that we can do user I/O via firmware in flash */
  sync() ;
  *flashBasePtr = 0xffffffff;

  /* a write error? */
  if ((status & 0x10101010) == 0x10101010) {
    status = FALSE ;
  } else 
    status = TRUE ;

  return status ;
}

void delete_flash_block(int blockno) 
{
  unsigned int status ;
  unsigned int offset = (blockno * flash_block_size) ;
  volatile unsigned int *flashIntP = (unsigned int *) (flash_base + offset);
  volatile unsigned int *flashIntBase = (unsigned int *)flash_base ;

  /* write delete block command followed by confirm */
  sync() ;
  *flashIntP = 0x20202020 ; 
  sync() ;
  *flashIntP = 0xd0d0d0d0 ;

  /* check the status */
  do {
    sync() ;
    *flashIntP = 0x70707070 ;
    status = *flashIntP;
  } while ((status & 0x80808080) != 0x80808080) ;
  if ((status & 0x20202020) == 0x20202020) {
    /* Make sure the flash is back in normal mode so 
       that we can do user I/O via firmware in flash */
    sync() ;
    *flashIntBase = 0xffffffff;
    return ;
  }
  /* Make sure the flash is back in normal mode so 
     that we can do user I/O via firmware in flash */
  
  sync() ;
  *flashIntBase = 0xffffffff ;

  blocks[blockno] = (char) -1 ;
}

