//
// Copyright (C) 1997 by
// Digital Equipment Corporation, Maynard, Massachusetts.
// All rights reserved.
//
// This software is furnished under a license and may be used and copied
// only  in  accordance  of  the  terms  of  such  license  and with the
// inclusion of the above copyright notice. This software or  any  other
// copies thereof may not be provided or otherwise made available to any
// other person.  No title to and  ownership of the  software is  hereby
// transferred.
//
// The information in this software is  subject to change without notice
// and  should  not  be  construed  as a commitment by Digital Equipment
// Corporation.
//
// Digital assumes no responsibility for the use  or  reliability of its
// software on equipment which is not supplied by Digital.
//
//
// Abstract:    
//
// Author:      
//
// Revision:    
//
// Modifications:
//
//     $History: $
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "platform.h"
#ifndef ANGEL
#include "cdef.h"
#endif
#include "fmu.h"
#ifndef ANGEL
#include "pciproto.h"
#endif

struct aif_header {
  unsigned int decompress ;
  unsigned int SelfReloc ;
  unsigned int DBGinit ;
  unsigned int EntryPoint ;
  unsigned int Exit ;
  unsigned int ROSize ;
  unsigned int RWSize ;
  unsigned int DebugSize ;
  unsigned int ZeroInitSize ;
  unsigned int ImageDebugType ;
  unsigned int ImageBase ;
} typedef aif_t ;

/* NOP instruction */
#define NOP      0xe1a00000 
/* bitmask to check for BL instruction */
#define BL       0x0a000000

/* Macro to create branch instructions */
#define BINSTR(offset) (0xeb000000+(((offset)-8)>>2))

#define NOBOOTMASK 1

struct image_header {
  int            Type ;
  unsigned char  Number ;
  unsigned char  Sig[3] ;
  int            Map ;
  int            Checksum ;
  int            Length ;
  unsigned char  Name[16]  ;
  unsigned int   Bootflags ; 
  unsigned char  Reserved[24] ;
} typedef image_header_t ;

#define IMAGE_HEADER_SIZE 64 

static struct image_header *header = NULL ;

/*
 *  Global variables
 */
unsigned volatile char *flash_base = NULL ;
extern unsigned int flash_block_size ;

char blocks[NUMBER_OF_BLOCKS] ;

#define BLOCK_IN_USE(n) (blocks[n] != ((char) -1))

#ifndef ANGEL
deviceInfo dInfo ;
#endif

/*
 *  Externals.
 */
extern unsigned int read_flash_dword(int offset) ;
extern int write_flash_dword(int offset, unsigned int data) ;
extern void delete_flash_block(int blockno) ;
extern int check_flash_id(volatile void *flash) ;

/*
 *  forward declarations.
 */
static void read_flash_header(int blockno) ;
static int write_flash_header(int offset, aif_t *aif, int unique, char *name, int map, int size, int file_checksum, int noboot);
static void print_flash_header(struct image_header *header) ;
static int valid_flash_header(struct image_header *header) ;

static void listAction(void) ;
static void listBlocksAction(void) ;
static void testBlockAction(void) ;
static void eraseAction(void) ;
static void delete_blockAction(void) ;
static void delete_allAction(void) ;
static void programAction(void) ;
static void readAction(void) ;
static void quitAction(void) ;
static void helpAction(void) ;

static int find_room(int blocks, int start, int * map) ;
static int find_image(int imageno) ;
static void scan_blocks(void) ;

long fsize(FILE *file)
{
  long offset ;

  fseek(file, 0, SEEK_END) ;
  offset = ftell(file) ;
  fseek(file, 0, SEEK_SET) ;

  return offset ;
}

/* ---------- M A K E     A N G E L   B A S E D  I M A G E   FA S T E R ------------*/
#ifdef ANGEL

static char *fileCache = NULL ;
static int fileCacheOffset = 0 ;

FILE *MyFopen(char *file, char *mode) 
{
  FILE *dataFile ;
  int size ;
  int i ;
  
  /* the the world */
  printf("Caching the file in memory\n") ;

  /* initialize */
  fileCacheOffset = 0 ;

  /* read the data file into a cached lump of memory */
  dataFile = fopen(file, mode) ;
  if (dataFile != NULL) {
    size = (int)fsize(dataFile) ;
    fileCache = (char *)malloc(size) ;
    if (fileCache == NULL) {
      fclose(dataFile) ;
      dataFile = NULL ;
    } else {
      i = 0 ;
      while (i < size) {
	int blockSize ;

	if (size - i < 512) 
	  blockSize = size - i ;
	else
	  blockSize = 512 ;

	if (fread(&fileCache[i], blockSize, 1, dataFile) == 0) {
	  printf("ERROR: reading data file\n") ;
	  fclose(dataFile) ;
	  dataFile = NULL ;
	  break ;
	} else {
	  i += blockSize ;
	}
      }
    }
  }

  /* return the file descriptor */
  return dataFile ;
}

int MyFseek(FILE *file, int offset, int flags)
{
  fileCacheOffset = offset ;
  return 0 ;
}

int MyFgetc(FILE *file) 
{
  return (int)fileCache[fileCacheOffset++] ;
}

int MyFclose(FILE *file) 
{
  fclose(file) ;
  fileCacheOffset = 0 ;
  if (fileCache) free(fileCache) ;
  fileCache = NULL ;
  return 0 ;
}

#define __fopen(file, mode) MyFopen(file, mode) 
#define __fseek(file, offset, flags) MyFseek(file, offset, flags) 
#define __fgetc(file) MyFgetc(file) 
#define __fclose(file) MyFclose(file) 

#else

#define __fopen(file, flags) fopen(file, flags) 
#define __fseek(file, offset, flags) fseek(file, offset, flags) 
#define __fgetc(file) fgetc(file) 
#define __fclose(file) fclose(file) 

#endif
/* ---------- P A R S I N G    &    A C T I O N --- R O U T I N E S ----------------*/

static int yes_no(void)
{
  char c ;

  printf("Do you really want to do this (y/N)? ") ;
  c = getchar() ;
  /* Ignore the rest of the line */
  if(c != '\n') while(getchar() != '\n');
  if (tolower(c) == 'y') 
    return TRUE ;
  else
    return FALSE ;
}

struct command {
  char *name;
  char *alias;
  char *help_args;
  char *help_text;
  void (*action_rtn)(void);
};

static struct command commands[] = {
  {"List", "Dir",
     "", 
     "List images in flash",
     &listAction},
  {"ListBlocks", "",
     "", 
     "List how each Flash block is being used",
     &listBlocksAction},
  {"TestBlock", "",
     "<block-number>",
     "Write a test pattern to a particular flash block",
     &testBlockAction},
  {"Delete", "Erase",
     "<image-number>",
     "Delete an image in flash",
     &eraseAction},
  {"DeleteBlock", "",
     "<block-number>",
     "Deletes a block that appears not to be in an image",
     &delete_blockAction},
  {"DeleteAll", "",
     "",
     "Deletes all blocks except block 0",
     &delete_allAction},
  {"Program", "",
     "<image-number> <image-name> <file-name> [<block-number>] [noboot]",
       "Program the given image into flash",
       &programAction},
  {"Read", "",
     "<image-number> <file-name>",
       "Read the given image from flash into a file",
       &readAction},
  {"Quit", "Exit",
     "",
     "Quit",
     &quitAction},
/*
 *  Help
 */
  {"Help", "?",
     "",
     "Print this help text", 
     &helpAction},
};

#define NUM_FMU_COMMANDS (sizeof(commands) / sizeof(struct command))

#define MAX_TOKENS 10
char *tokens[MAX_TOKENS];
int ntokens = 0;

static char *__PULL()
{
  char *value = NULL;

  if (ntokens > 0) {
    int i;

    value = tokens[0];
    for (i = 1; i < ntokens; i++) tokens[i-1] = tokens[i];
    ntokens--;
  }
  return value;
}

#define __ADD(value) tokens[ntokens++] = value

/*  List all of the images in flash in ascending image number order */
static void listAction(void)
{
    int i, j, k ;
    int blockno ;
    int checksum;
    int image_offset, flash_offset;
    int in_use_map = 0;

    printf("Listing images in Flash\n");

    /* look for every possible image number */
    for (i = 0 ; i < 31 ; i++) {
	blockno = find_image(i) ;
	if (blockno >= 0)  {
	    read_flash_header(blockno) ;
	    print_flash_header(header) ;
	    /* Check for dual allocation of blocks */
	    if((in_use_map & header->Map) != 0) {
		printf("ERROR: Some blocks in this image have are also allocated to earlier images");
	    }
	    in_use_map |= header->Map;
	    /* Check the checksum */
	    image_offset = 0 ;
	    checksum = 0 ;
	    for(j=0;j< NUMBER_OF_BLOCKS; j++) {
		if(blocks[j] == header->Number) {
		    flash_offset = j * flash_block_size ; 
		    for(k=0; k < flash_block_size; k+=4) {
			if(image_offset >= header->Length) {
			    break;
			}
			checksum += read_flash_dword(flash_offset) ;
			flash_offset += 4 ;
			image_offset += 4 ;
		    }
		}
	    }
	    if(checksum != 0) {
		printf("Checksum error; image checksum is 0x%x, should be 0\n",checksum);
	    }
	}
    }
}

/* list how each flash block is being used */
static void listBlocksAction(void) 
{
  int offset ;
  int i ;
  int blockno ;

  /* look at every block in turn */
  for (blockno = 0; blockno < NUMBER_OF_BLOCKS; blockno++) {
    offset = blockno * flash_block_size ;
    printf("%2d: ", blockno) ;
    if BLOCK_IN_USE(blockno) 
      printf("(Image %2d) ", blocks[blockno]) ;
    else 
      printf("(Unused)   ") ;
    for (i = 0; i < 5 ; i++) {
      printf("0x%08x ", read_flash_dword(offset)) ;
      offset += 4 ;
    }
    printf("\n") ;
  }
}

/* write and read a test pattern into block N */
static void testBlockAction(void) 
{
  int blockno ;
  char *token ;
  unsigned char data = 0 ;
  int i, j, w ;
  int offset ;
  int ecount ;

  /* did the user supply a block number? */
  token = __PULL() ;
  if (token == NULL) {
    printf("ERROR: you should supply a block number to test\n") ;
    return ;
  }

  /* ...and is it valid? */
  blockno = atoi(token) ;
  if ((blockno > (NUMBER_OF_BLOCKS - 1)) ||
      (blockno < 0)) {
    printf("ERROR: invalid block number\n") ;
    return ;
  }

  /* can it be deleted? */
  if (BLOCK_IN_USE(blockno))  {
    printf("ERROR: that block is not free (image # %d)\n",
	   blocks[blockno]) ;
    return ;
  }

  /* does the user really want to? */
  if (!yes_no()) return ;

  /* delete it before testing it */
  printf("deleting block %d\n", blockno) ;
  delete_flash_block(blockno) ;

  /* write and read a test pattern to the block */
  printf("Writing test pattern to block %d\n", blockno) ;
  data = 0 ;
  offset = blockno * flash_block_size ;
  for (i = 0 ; i < flash_block_size ; i += 4) {
    w = 0 ;
    for (j = 0 ; j < 4 ; j++) {
      if (data == 0xff) data = 0 ;
      w |= (data << (j * 8)) ;
      data++ ;
    }
    write_flash_dword(offset, w) ;
    offset += 4 ;
  }

  printf("Reading test pattern from block %d\n", blockno) ;
  data = 0 ;
  offset = blockno * flash_block_size ;
  for (i = 0 ; i < flash_block_size ; i += 4) {
    w = 0 ;
    for (j = 0 ; j < 4 ; j++) {
      if (data == 0xff) data = 0 ;
      w |= (data << (j * 8)) ;
      data++ ;
    }
    if (read_flash_dword(offset) != w) {
      ecount++ ;
      if (ecount < 5) {
	printf("ERROR: test pattern does not match (offset = 0x%x)\n",
	       offset) ;
	printf("\tValue is 0x%08x, should be 0x%08x\n", read_flash_dword(offset),
	       w) ;
      }
    }
    offset += 4 ;
  }

  printf("Flash test of block %d worked\n", blockno) ;
}

/* delete an image in flash */
static void eraseAction(void) 
{
  char *token ;
  int imageno ;
  int blockno ;
  int checksum;
  int image_offset, flash_offset;
  int j,k;

  /* did the user supply an image number? */
  token = __PULL() ;
  if (token == NULL) {
    printf("ERROR: you should supply an image number to delete\n") ;
    return ;
  }

  /* ...and is it valid? */
  imageno = atoi(token) ;
  if ((imageno > 31) || (imageno < 0)) {
    printf("ERROR: invalid image number\n") ;
    return ;
  }

  /* can I find it? */
  blockno = find_image(imageno) ;
  if (blockno < 0) {
    printf("ERROR: no such image\n") ;
    return ;
  }

#ifdef ANGEL
  /* cannot delete image starting in flash block 0 */
  if (blockno == 0) { 
    printf("ERROR: cannot delete image in flash block 0\n") ;
    return ;
  }
#endif

  /* does the user really want to? */
  if (!yes_no()) return ;

  /* Check the checksum */
  checksum = 0;
  image_offset = 0;
  for(j=0;j< NUMBER_OF_BLOCKS; j++) {
      if(blocks[j] == header->Number) {
	  flash_offset = j * flash_block_size;
	  for(k=0; k < flash_block_size; k+=4) {
	      if(image_offset >= header->Length) {
		  break;
	      }
	      image_offset+=4;
	      checksum += read_flash_dword(flash_offset) ;
	      flash_offset += 4 ; 
	  }
      }
  }
  if(checksum !=0) {
      /* Bad checksum, delete only the first block to avoid corruption */
      printf("WARNING: Bad checksum; only deleting first block (block no %d)\n", blockno);
      delete_flash_block(blockno);
  } else {
      /* delete it */
      printf("Deleting flash blocks: ") ;
      while ((blocks[blockno] == imageno) && (blockno < NUMBER_OF_BLOCKS)) {
	  printf("%d ", blockno) ;
	  delete_flash_block(blockno) ;
	  blockno++;
      }
  }
  printf("\n") ;
  scan_blocks();

}

/* delete a block in flash, so long as it is not in an image */
static void delete_blockAction(void) 
{
  char *token ;
  int blockno ;

  /* did the user supply a block number? */
  token = __PULL() ;
  if (token == NULL) {
    printf("ERROR: you should supply a block number to delete\n") ;
    return ;
  }

  /* ...and is it valid? */
  blockno = atoi(token) ;
  if ((blockno > (NUMBER_OF_BLOCKS - 1)) ||
      (blockno < 0)) {
    printf("ERROR: invalid block number\n") ;
    return ;
  }

  /* can it be deleted? */
  if (BLOCK_IN_USE(blockno))  {
    printf("ERROR: that block is not free (image # %d)\n",
	   blocks[blockno]) ;
    return ;
  }

  /* does the user really want to? */
  if (!yes_no()) return ;

  printf("Delete flash block %d\n", blockno) ;
  delete_flash_block(blockno) ;
  scan_blocks();

}

/* delete all blocks except block 0 */
static void delete_allAction(void) 
{
  int i ;

  /* does the user really want to do this? */
  if (!yes_no()) return ;

  printf("Deleting flash blocks: ") ;
  for (i = 1 ; i < NUMBER_OF_BLOCKS ; i++ )  {
    printf("%d ", i) ;
    delete_flash_block(i) ;
  }
  printf("\n") ;
}

/* read an AIF header and check it */
static int read_aif_header(FILE *file, aif_t *aif) 
{
  unsigned char data ;
  int i ;

  /* Read the AIF image header */
  for (i = 0 ; i < sizeof(aif_t) ; i++) {
    data = __fgetc(file) ;
    if (data == EOF) {
      printf("ERROR: reading file\n") ;
      __fclose(file) ;
      return -1 ;
    }
    *((char *)(aif) + i) = data ;
  }
  
  /* reset the file to offset 0 */
  __fseek(file, 0, SEEK_SET) ;

  /* return success */
  return 0 ;
}

/* program an image into flash */
static void program_flash(int imageno, 
			  char *name, 
			  int blockno, 
			  char *filename, 
			  int noboot) 
{
  FILE *file ;
  int blocks ;
  aif_t aif ;
  int i, j, offset ;
  int map ;
  int size ;
  unsigned int data ;
  int checksum;

  /* open the file */
  file = __fopen(filename, "rb") ;
  if (file == NULL) {
    printf("ERROR: failed to open file\n") ;
    return ;
  }

  /* is there room? */
  size = (int)fsize(file) ;
  blocks = (size + IMAGE_HEADER_SIZE) / flash_block_size ;
  if (((size + IMAGE_HEADER_SIZE) % flash_block_size) != 0) blocks++ ;
  blockno = find_room(blocks, blockno, &map) ;
  if (blockno < 0) {
    printf("ERROR: no room for the image in flash\n") ;
    return ;
  }

  /* read the aif header into memory (to play with) */
  if (read_aif_header(file, &aif) < 0)  {
    __fclose(file) ;
    return ; 
  }

  /* tell the world what is going on */
  if (blocks == 1) 
    printf("Writing %s into flash block %d",
	   filename, blockno) ;
  else {
      printf("Writing %s into flash: blocks ",
	     filename) ;
      for (i = blockno ; i < NUMBER_OF_BLOCKS; i++) {
	  if(map & (1<<i)) { 
	      printf("%d ", i) ;
	  }
      }
  }

  /* delete the flash blocks that this image will take up */
  printf("\nDeleting blocks ready to program:\n") ;
  for (i = blockno ; i < NUMBER_OF_BLOCKS; i++) {
      if(map & (1<<i)) { 
	  printf("Deleting block %d \n", i) ;
	  delete_flash_block(i) ;
      }
  }

  /* Calculate the file checksum. The checksum is simply the (little
     endian) 32 bit sum of the 32 bit words of the file. 
     Overflow is ignored, and excess bytes in the last word are treated 
     as 0xff (since this is the value of unprogrammed bytes of a flash).
     The checksum of the header will added in in write_flash_header */
  printf("Calculating checksum\n");
  checksum=0;
  __fseek(file, 0, SEEK_SET) ;
  for(i=0;i<size;i++) {
      checksum += __fgetc(file) << ((i%4)*8);
  }
  for(i=((size-1)%4)+1;i<4;i++) {
      checksum += 0xff << i*8;
  }

  /* build an image header and write it to flash */
  offset = blockno * flash_block_size ;
  offset = write_flash_header(offset, &aif, imageno, name, map,
			      size, checksum, noboot) ;
  if  (offset < 0) {
    __fclose(file) ;
    return ;
  }

  /* write the whole file into flash */
  __fseek(file, 0, SEEK_SET) ;
  printf("Writing image file\n") ;
  putchar('#') ;
  for (i = 0; i < size; i += 4) {
    if ((((unsigned int)offset) % 512) == 0) putchar('#') ;
    if ((((unsigned int)offset) % flash_block_size) == 0) {
	blockno++;
	while(!(map & (1 << blockno))) {
	    blockno++;
	    offset+=flash_block_size;
	}
    }
    data = 0 ;
    for (j = 0 ; j < 4 ; j++) {
      data |= (__fgetc(file) << (j * 8)) ;
    }
    write_flash_dword(offset, data) ;
    offset += 4 ;
  }
  putchar('\n') ;

  /* close the file */
  __fclose(file) ;

/* 
 * finally, re-scan the flash blocks 
 */
  scan_blocks() ;
}

/* program an image into flash */
static void programAction(void) 
{
  char *token ;
  int imageno ;
  int blockno = -1 ;
  char *imagename ;
  char *filename ;
  int noboot = FALSE;

/*
 *  Get all the arguments.
 */
  /* image number supplied? */
  token = __PULL() ;
  if (token == NULL) {
    printf("ERROR: you should supply an image number\n") ;
    return ;
  }
  imageno = atoi(token) ;

  /* is it a valid (unused) image number? */
  if (find_image(imageno) >= 0) {
    printf("ERROR: image number already in use\n") ;
    return ;
  }

  /* image name supplied? */
  token = __PULL() ;
  if (token == NULL) {
    printf("ERROR: you should supply an image name\n") ;
    return ;
  }
  imagename = token ;
  
  /* file name supplied? */
  token = __PULL() ;
  if (token == NULL) {
    printf("ERROR: you should supply a file name\n") ;
    return ;
  }
  filename = token ;

  /* block number supplied */
  token = __PULL() ;
  if (token != NULL) {
      if(isdigit(*token)) {
	  blockno = atoi(token) ;
	  if ((blockno < 0) || (blockno > (NUMBER_OF_BLOCKS -1))) {
	      printf("ERROR: invalid block number\n") ;
	      return ;
	  }
	  if (BLOCK_IN_USE(blockno)) {
	      printf("ERROR: block in use by image %d\n", blocks[blockno]) ;
	      return ;
	  }
	  token = __PULL() ;
      }
  }
  if(token != NULL) {
      char *tp=token;
      while(*tp!=0) {
	  *tp=tolower(*tp);
	  tp++;
      }
      if(strcmp(token,"noboot") == 0) {
	  noboot=TRUE;
      } else {
	  printf("ERROR: bad option %s\n", token);
	  return;
      }
  }

/*
 * program the image into flash.
 */ 
  program_flash(imageno, imagename, blockno, filename, noboot) ;
}

/* Read an image from flash */
static void readAction(void) 
{
FILE	*file ;
char	*token ;
char	*filename ;
int	 imageno ;
int	 blockno = -1 ;
int	 flash, checksum ;
int	 image_offset, flash_offset ;
int	 j, k ;

	/*
	 *  Get all the arguments.
	 */
	/* image number supplied? */
	token = __PULL() ;
	if ((token == NULL) || (isdigit(*token) == 0)) {
		printf("ERROR: you must supply an image number\n") ;
		return ;
	}
	imageno = atoi(token) ;

	/* is it a valid (used) image number? */
	blockno = find_image(imageno) ;
	if (blockno < 0) {
		printf("ERROR: image %d is empty\n", imageno) ;
		return ;
	}

	/* file name supplied? */
	token = __PULL() ;
	if (token == NULL) {
	printf("ERROR: you must supply a file name\n") ;
	return ;
	}
	filename = token ;

	token = __PULL() ;
	/* Any stray paramters? */
	if(token != NULL) {
		printf("ERROR: bad option %s\n", token);
		return;
	}

	/* open the file */
	file = fopen(filename, "rb") ;
	if (file != NULL) {
		printf("WARNING: this will overwrite the file %s\n", filename) ;
		if (!yes_no())
			return ;
	}

	file = fopen(filename, "wb") ;
	if (file == NULL) {
		printf("ERROR: failed to open file\n") ;
		return ;
	}

	image_offset = 0 ;
	checksum = 0 ;
	read_flash_header(blockno) ;

	if (!valid_flash_header(header)) {
		printf("ERROR: image header is corrupt\n") ;
		return ;
	}
	for (j = 0; j < NUMBER_OF_BLOCKS; j++) {
		if (blocks[j] == header->Number) {
			flash_offset = j * flash_block_size ; 
			for (k = 0; k < flash_block_size; k += 4) {
				if (image_offset >= header->Length) {
					break;
				}
				flash = read_flash_dword(flash_offset) ;
				checksum += flash ;
				if ( image_offset >= IMAGE_HEADER_SIZE) {
					fputc((unsigned char)((flash >> 24) & 0xFF), file);
					fputc((unsigned char)((flash >> 16) & 0xFF), file);
					fputc((unsigned char)((flash >> 8) & 0xFF), file);
					fputc((unsigned char)(flash & 0xFF), file);
				}
				flash_offset += 4 ;
				image_offset += 4 ;
			}
		}
	}

	__fclose(file) ;

	if(checksum != 0) {
		printf("Checksum error; image checksum is 0x%x, should be 0\n",
			checksum);
	}
}

/* let's get outa here! */
static void quitAction(void) 
{
  exit(0);
}

static void helpAction(void) 
{
#define COMMAND_SIZE 20
  int i, count;
  printf("FMU command summary:\n\n");

  for (i = 0; i < NUM_FMU_COMMANDS; i++) {
    printf("%s ", commands[i].name);
    count = strlen(commands[i].name);
    count++;
    printf("%s ", commands[i].help_args);
    count += strlen(commands[i].help_args);
    count++;
    if (count < COMMAND_SIZE) {
      count = COMMAND_SIZE - count;
    } else {
      printf("\n");
      count = COMMAND_SIZE;
    }
    while(count--) printf(" ");
    printf("- %s\n", commands[i].help_text); 
  }
}

static int partial_compare(char *token, char *command) 
{
  int len ;
  int i ;

  if (strlen(command) < strlen(token)) return FALSE ;
  len = strlen(token) ;

  for (i = 0 ; i < len ; i++)  
    if (tolower(*token++) != tolower(*command++)) return FALSE ;

  return TRUE ;
}

static int compare(char *token, char *command) 
{
  int len ;
  int i ;

  if (strlen(command) != strlen(token)) return FALSE ;
  len = strlen(token) ;

  for (i = 0 ; i < len ; i++)  
    if (tolower(*token++) != tolower(*command++)) return FALSE ;

  return TRUE ;
}

static void parse_command(void)
{
    char *token;
    int i;
    int found = 0;
    int index = -1 ;

    /* now work out what the command was from the first token */
    token = __PULL();
    if (token != NULL) {
      /* first, look for an absolute match */
      for (i = 0; i < NUM_FMU_COMMANDS; i++) {
	if (compare(token, commands[i].name) == TRUE ||
	    compare(token, commands[i].alias) == TRUE) {
	  (commands[i].action_rtn)();
	  return ;
	}
      }
      /* now, look for a partial absolute match */
      for (i = 0; i < NUM_FMU_COMMANDS; i++) {
	if (partial_compare(token, commands[i].name) == TRUE ||
	    partial_compare(token, commands[i].alias) == TRUE) {
	  found++ ;
	  index = i ;
	}
      }
      if (found == 0) 
	printf("FMU: unknown command, try typing Help\n");
      else {
	if (found > 1) 
	  printf("ERROR: more than one command matches, try typing Help\n") ;
	else 
	  (commands[index].action_rtn)();
      }
    }
  }

static void parse_command_string(char *string)
{
    char *token;

    /* Grab all of the command tokens */
    ntokens = 0;
    token = strtok(string, " \t");
    while(token != NULL) {
      /* treat ';' as end of line, new command */
      if (strcmp(token, ";") == 0) {
	parse_command();
	ntokens = 0;
      } else {
	__ADD(token);
      }
      token = strtok(NULL, " \t");
    }
    /* now that we've found it, parse it. */
    parse_command();

}

/* ---------- I M A G E --- S U P P O R T --- R O U T I N E S ----------------*/

/* given the size of the image (in bytes), find room for it */
static int find_room(int number, int first, int * map) 
{
    int last ;
    int count ;
    int header_block;

    *map=0;

    /* find a sequence of N free blocks */
    if (first >= 0) {
	/* The user has specified a block number so they must be sequential
	   from that block */
	for(last = first; last < first+number; last++) {
	    if(BLOCK_IN_USE(last)) {
		/* Can't put it where the user specified, give up */
		return -1;
	    } else {
		*map |= (1<<last);
	    }
	}
	return first;
    } else {
	count=0;
	for(last=0; last < NUMBER_OF_BLOCKS; last++) {
	    if(!BLOCK_IN_USE(last)) {
		*map  |= (1<<last);
		if(count==0) {
		    header_block=last;
		}
		count++;
		if(count==number) {
		    return header_block;
		}
	    }
	}
	
	/* no room */
	return -1 ; 
    }
}

static int find_image(int imageno)
{
  int i ;

  for (i = 0 ; i < NUMBER_OF_BLOCKS ; i++) {
    if (blocks[i] == imageno) return i ;
  }
  
  return -1 ;
}

static int valid_flash_header(struct image_header *header) 
{
  /* perform a series of tests to validate the header */
  /* valid signature (0x55, 0xaa, 0x00)? */
  if (header->Sig[0] != 0x55) return FALSE ;
  if (header->Sig[1] != 0xaa) return FALSE ;
  if (header->Sig[2] != 0x00) return FALSE ;

  /* return happily */
  return TRUE ;
}

static void read_flash_header(int blockno)
{
int		 i ;
int		 offset = blockno * flash_block_size ;
unsigned int	*buffer = (unsigned int *)header ;

  for (i = 0 ; i < sizeof(image_header_t) ; i += 4 ) {
	*buffer++ = read_flash_dword(offset) ;
	offset += 4 ;
  }
}

static void print_flash_header(struct image_header *header)
{
  int i ;

  printf("Image %d \"", header->Number) ;
  for (i = 0; i < sizeof(header->Name); i++) {
      printf("%c", header->Name[i]) ;
  }
  printf("\" Length %d bytes", header->Length) ;
  printf(", Map 0x%08x", header->Map) ;
  if((header->Bootflags) & NOBOOTMASK) {
      printf(" Noboot");
  }
  printf("\n");
}

static void scan_blocks(void) 
{
    int j ;
    int blockno ;

    printf("Scanning Flash blocks for usage\n") ;

    /* assume the worst */
    for (blockno = 0 ; blockno < NUMBER_OF_BLOCKS ; blockno++)  
	blocks[blockno] = (char) -1 ;

    /* look at every block in turn, trying to find a header. */
    for (blockno = 0 ; blockno < NUMBER_OF_BLOCKS ; blockno++)  {
	read_flash_header(blockno) ;
	if (valid_flash_header(header) && (blocks[blockno] == (char) -1)) {
	    for (j = 0 ; j < NUMBER_OF_BLOCKS ; j++) {
		if (((header->Map) & (1 << j)) != 0) {
		    if(blocks[j] == (char) -1){
			blocks[j] = header->Number ;
		    } else {
			printf("ERROR: Two images own block %d\n",j);
		    }
		}
	    }
	}
    }
}

static int write_flash_header(int offset, aif_t *aif, int unique, char *name, int map, int size, int file_checksum, int noboot) 
{
  int count, i;
  unsigned int *data = NULL ;
  int pad = 0 ;
  int checksum;
  int * header_p;
  

  /* tell the world */
  printf("Writing flash image header\n") ;

  /* zero out the header */
  memset((char *)header, 0, sizeof(image_header_t))  ;

  /* work out what sort of image it is */
  if ((aif->EntryPoint & BL) == BL) {
    printf("Image is Executable AIF file\n") ;
    printf("Image will execute from Flash\n") ;
    /* Create a BL to the start of the aif header (The aif header itself is executable) */
    header->Type = BINSTR(0x40)  ; 
  } else {
      printf("Image is non-executable AIF file\n") ;
      pad = IMAGE_HEADER_SIZE - sizeof(image_header_t) ;
      if((aif->ImageBase > FLASH_BASE) && (aif->ImageBase < FLASH_END)) {
	  /* The base address is within the EPROM or FLASH area. This
	   * only makes sense if the image is to be executed from FLASH.
	   */
	  
	  /* Check that the bottom bits of the base address are correct */
	  if((aif->ImageBase & 0x00ffffff) != offset+0xc0) {
	      printf("ERROR: image base address is in ROM but does not match address of flash block\n") ;
	      printf("       Image base address = 0x%x\n",aif->ImageBase);
	      printf("       Expected start of image = 0x??%06x\n",offset+0xc0);
	      return -1;
	  }
	  printf("Image will execute from Flash\n") ;
	  /* Put in a branch to the entry point */
	  header->Type= BINSTR((0xc0 + aif->EntryPoint));
      } else {
	  printf("The bootloader will copy this image to %x before executing it\n",aif->ImageBase);
	      printf("       Eprom is from 0x%x to 0x%x\n", FLASH_BASE, FLASH_END) ;
	  /* The bootloader will relocate the image */
	  header->Type = NOP ;
      }
  }

  header->Number = unique ;

  header->Sig[0] = 0x55 ;
  header->Sig[1] = 0xaa ;
  header->Sig[2] = 0x00 ;

  header->Map = map ;
  header->Bootflags = noboot & NOBOOTMASK;

  /* it has a name */
  for (i = 0 ; i < sizeof(header->Name); i++) header->Name[i] = ' ';
  for (i = 0 ; i < strlen(name) ; i++) header->Name[i] = name[i] ;

  /* it's size */
  header->Length = size + IMAGE_HEADER_SIZE ;

  /* work out the checksum */
  header->Checksum = 0 ;

  checksum=file_checksum;
  header_p = (int *)header;
  for(i=0; i<sizeof(image_header_t); i+=4) {
      checksum += *(header_p++);
  }
  /* Negate the checksum so that the final checksum of the whole image is 0 */
  header->Checksum = -checksum;
  
  /* write it out */
  count = sizeof(image_header_t) ;
  data = (unsigned int *)header ;
  do {
    if (!write_flash_dword(offset, *data++)) {
      printf("ERROR: writing flash header at offset 0x%x (%d)\n",
	     offset, offset) ;
      return -1 ;
    }
    offset += 4 ;
    count  -= 4 ;
  } while (count > 0) ;

  printf("padding is %d bytes\n", pad) ;
  /* pad it out */
  while (pad) {
    if (!write_flash_dword(offset++, 0)) {
      printf("ERROR: writing flash header at offset 0x%x (%d)\n",
	     offset, offset) ;
      return -1 ;
    }
    pad -= 4 ;
  }
     
  /* return successfully */
  return offset ;
}

/* ------------------------ M A I N ---------------------------- */
#define fmuBanner "Digital EBSA-285 Flash Management Utility [v1.4]"

#ifdef ANGEL

int main(void)
{
  extern unsigned int EnterSvcMode(void) ;
  extern void DisableCaches(void) ;
  char command[80] ;
  unsigned int tmp ;

  /* tell the world just who we are */
  printf(fmuBanner);
  printf(" (Angel)\n") ;

  /* get into SVC mode */
  printf("Entering SVC mode") ;
  tmp = EnterSvcMode() ;
  printf("...Done\n") ;

  /* Disable the caches */
  printf("Disabling caches") ;
  DisableCaches() ;
  printf("...Done\n") ;

  /* try and find the flash */
  printf("Searching for flash device\n") ;
  if (check_flash_id((unsigned char *)FLASH_BASE) == TRUE) {
    flash_base = (unsigned char *)FLASH_BASE ;
  } else {
    printf("Failed to find flash\n") ;
    exit(1) ;
  }
  printf("Flash found at 0x%p (%d blocks of size 0x%x)\n", 
	 flash_base, NUMBER_OF_BLOCKS, flash_block_size) ;

  header = (struct image_header *)malloc(IMAGE_HEADER_SIZE) ;
  if (header == NULL) {
    printf("ERROR: failed to allocate memory\n") ;
    exit(1) ;
  }

  scan_blocks() ;

  /* now just get strings from the user and parse them */
  printf("FMU> ");
  while (gets(command) != 0) {
    parse_command_string(command) ;
    printf("FMU> ") ;
  }

  /* return normally */
  exit(0) ;
}
#else
void main(void)
{    
  char command[80] ;
  biosInfo bInfo;
  devInfo device;
#if defined SCAN_VALID  
  unsigned int x;
#endif  
  unsigned int done=0,problem=0;
  PU32 tPntr;

  /* tell the world just who we are */
  printf(fmuBanner);
  printf(" (DOS)\n") ;

  //
  // make sure that the system supports PCI bios calls.
  //
  if (checkPciBios(&bInfo)==MSG_FAILURE) {
    printf("\nouch ! PCI BIOS not detected !\n");
    exit(1);
  }

  if (bInfo.hwMech == 0) {
    printf("ouch ! problem with Config Mechanism detection !\n");
    exit(1);
  }

  printf("PCI Config Mechanism: #%d, BIOS version: %d.%d\n",
                                 bInfo.hwMech&3, bInfo.major,bInfo.minor);

  //
  // try and find device
  //
  printf("Searching for Digital DC21285 device\r");

  if (findDevice(0x1011,0x1065,0,&device) == MSG_FAILURE) {
    printf("unable to locate any Digital DC21285 devices\n");
    exit(1);
  }

  dInfo.bus = device.bus;
  dInfo.slot = device.slot;
  dInfo.function = device.function;
  dInfo.bar0Phys = incfgd(0,dInfo.bus,dInfo.slot,dInfo.function,0x10);

  printf("Found Digital DC21285 device on bus:%d, slot:%d\n",dInfo.bus,
          dInfo.slot);
  dInfo.romPhys = 0xB0000000;

  //
  // we need to map 2 areas of memory. The first will allow us to access
  // the 128 memory mapped CSR's for the rogue low byte address feature, 
  // the second will create a 16 MB area that we can map the flash ROM into.
  // Sooooo, this code will scan looking for 16 MB sections for us to use 
  // for ROM space. Chances are that this first location will work fine,
  // but we need to prepare for the worst.
  //
  while (!done) {  

    printf("scanning region 0x%08X\r",dInfo.romPhys);
    if (dpmiMap(dInfo.romPhys,0x1000000,(PU32)&dInfo.romVirt,0) == MSG_FAILURE) {
      printf("ouch ! unable to allocate DPMI memory to map ROM\n");
      exit(1);
    }

    tPntr = (PU32)dInfo.romVirt;

    //
    // since we are flying by the seat of our pants here, we need to make
    // sure that no other device is present. BIOS does not have a nice
    // call that would allocate non-existant memory space for us to map
    // our ROM to. So scan through the region expecting master aborts 
    // throughout.
    //
#if defined SCAN_VALID
    for (x=0;x<0x1000000/4;x++) {

      //
      // we didn't get a master abort
      //
      if (tPntr[x] != 0xFFFFFFFF) {

        //
        // set issue flag
        //
        problem = 1;

        //
        // bump our region to the next aligned 16 MB region
        //
        dInfo.romPhys += 0x1000000;

        //
        // free up the last allocated region
        //
        dpmiFree(dInfo.romVirt,0x1000000,0);
      }
    }
#endif

    //
    // if there were no problems, then we have a good region
    //
    if (problem == 0) 
      done = 1;
    else 

      //
      // else reset the issue flag
      //
      problem = 0;

    //
    // make sure that we don't fall off of the end of memory 
    // this is where the system ROM is.
    //
    if (dInfo.romPhys == 0xF0000000) {
      printf("ouch ! unable to locate available memory for ROM\n");
      exit(1);
    }   
  } 
  
  printf("Expansion ROM BAR mapped to physical memory at 0x%08X\n",dInfo.romPhys);

  //
  // write the Expansion ROM register with our region location
  //
  outcfgd(0,dInfo.bus,dInfo.slot,dInfo.function,0x30,dInfo.romPhys|1);    

  //
  // map the CSR's into memory space
  //
  if (dpmiMap(dInfo.bar0Phys,0x80,(PU32)&dInfo.bar0Virt,0) == MSG_FAILURE) {
    printf("ouch ! unable to allocate DPMI memory to map 21285 CSR's\n");
    exit(1);
  }
  /* try and find the flash */
  printf("Searching for flash device\n") ;

#ifdef REAL_DEVICE  
  if (check_flash_id((unsigned char *)dInfo.romVirt) == TRUE) {
    flash_base = (unsigned char *)dInfo.romVirt;
  }
  else {
    printf("Failed to find flash\n") ;
    exit(1) ;
  }
  printf("Flash found at 0x%p (%d blocks of size 0x%x)\n", 
        dInfo.romPhys, NUMBER_OF_BLOCKS, flash_block_size) ;

#else
  flash_base = (unsigned char *)dInfo.romVirt;

  printf("Flash found at 0x%p (%d blocks of size 0x%x)\n", 
        dInfo.romPhys, NUMBER_OF_BLOCKS, flash_block_size) ;
#endif

  header = (struct image_header *)malloc(IMAGE_HEADER_SIZE) ;
  if (header == NULL) {
    printf("ERROR: failed to allocate memory\n") ;
    exit(1) ;
  }

  /* scan the flash blocks for images */
  scan_blocks() ;

  /* now just get strings from the user and parse them */
  printf("FMU> ");
  while (gets(command) != 0) {
    parse_command_string(command) ;
    printf("FMU> ") ;
  }

  //
  // free the buffers
  //
  dpmiFree(dInfo.romVirt,0x1000000,0);
  dpmiFree(dInfo.bar0Virt,0x80,0);

  /* return normally */
  exit(0) ;
}
#endif
