//
// Copyright (C) 1997 by
// Digital Equipment Corporation, Maynard, Massachusetts.
// All rights reserved.
//
// This software is furnished under a license and may be used and copied
// only  in  accordance  of  the  terms  of  such  license  and with the
// inclusion of the above copyright notice. This software or  any  other
// copies thereof may not be provided or otherwise made available to any
// other person.  No title to and  ownership of the  software is  hereby
// transferred.
//
// The information in this software is  subject to change without notice
// and  should  not  be  construed  as a commitment by Digital Equipment
// Corporation.
//
// Digital assumes no responsibility for the use  or  reliability of its
// software on equipment which is not supplied by Digital.
//
//
// Abstract:    
//
// Author:      
//
// Revision:    
//
// Modifications:
//
//     $History: $
//
#include "platform.h"
#include "post.h"
#include "fmu.h"

/*
 *  Local definitions.
 */
#ifndef TRUE 
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0 
#endif

#ifndef NULL
#define NULL (void *)0
#endif

struct image_header {
  int            Type ;
  unsigned char  Number ;
  unsigned char  Sig[3] ;
  int            Map ;
  int            Checksum ;
  int            Length ;
  unsigned char  Name[16]  ;
  unsigned int   Bootflags ; 
  unsigned char  Reserved[24] ;
} typedef image_header_t ;

#define IMAGE_HEADER_SIZE 64 

static struct image_header staticHeader ;
static struct image_header *header = NULL ;

/*
 *  Global variables
 */

unsigned volatile char *flash_base = NULL ;
extern unsigned int flash_block_size ;
char blocks[NUMBER_OF_BLOCKS] ;

#define BLOCK_IN_USE(n) (blocks[n] != ((char) -1))

static void list_blocks(void) ;
static int test_block(int) ;
static void scan_blocks(void) ;

/*
 *  Externals
 */
extern unsigned int read_flash_dword(int offset) ;
extern int write_flash_dword(int offset, unsigned int data) ;
extern void delete_flash_block(int blockno) ;
extern int check_flash_id(volatile void *flash) ;

static int valid_flash_header(struct image_header *header) 
{
  /* perform a series of tests to validate the header */
  /* valid signature (0x55, 0xaa, 0x00)? */
  if (header->Sig[0] != 0x55) return FALSE ;
  if (header->Sig[1] != 0xaa) return FALSE ;
  if (header->Sig[2] != 0x00) return FALSE ;

  /* return happily */
  return TRUE ;
}

static void read_flash_header(int blockno)
{
  int i ;
  int offset = blockno * flash_block_size ;
  unsigned int *buffer = (unsigned int *)header ;

  for (i = 0 ; i < sizeof(image_header_t) ; i += 4 ) {
    *buffer++ = read_flash_dword(offset) ;
    offset += 4 ;
  }
}

static void scan_blocks(void) 
{
    int j ;
    int blockno ;

    PutString("Scanning Flash blocks for usage\n\r") ;

    /* assume the worst */
    for (blockno = 0 ; blockno < NUMBER_OF_BLOCKS ; blockno++)  
	blocks[blockno] = (char) -1 ;

    /* look at every block in turn, trying to find a header. */
    for (blockno = 0 ; blockno < NUMBER_OF_BLOCKS ; blockno++)  {
	read_flash_header(blockno) ;
	if (valid_flash_header(header) && (blocks[blockno] == (char) -1)) {
	    for (j = 0 ; j < NUMBER_OF_BLOCKS ; j++) {
		if (((header->Map) & (1 << j)) != 0) {
		    if(blocks[j] == (char) -1){
			blocks[j] = header->Number ;
		    }
		}
	    }
	}
    }
}

/* list how each flash block is being used */
static void list_blocks(void) 
{
  int offset ;
  int j ;
  int blockno ;
  unsigned int value ;

  PutString("Listing Flash Blocks\n\r") ;

  /* look at every block in turn */
  for (blockno = 0; blockno < NUMBER_OF_BLOCKS; blockno++) {
    offset = blockno * flash_block_size ;
    PutHex(blockno) ;
    if (BLOCK_IN_USE(blockno))
      PutString(" * ") ;
    else 
      PutString("   ") ;

    for (j = 0; j < 4 ; j++) {
      value = read_flash_dword(offset) ;
      PutHex(value) ; PutString(" ") ;
      offset += 4 ;
    }
    if (BLOCK_IN_USE(blockno)) {
      read_flash_header(blockno) ;
      PutString((char *)header->Name) ;
    }
    PutString("\n\r") ;
  }
}

/* write and read a test pattern into block N */
static int test_block(int blockno) 
{
  unsigned char data = 0 ;
  int i, j, w ;
  int offset ;
  int ecount ;

  /* write and read a test pattern to the block */
  PutString("Testing Block ") ;
  PutHex(blockno) ; PutString("\n\r") ;

  PutString("Writing test pattern\n\r") ;
  data = 0 ;
  offset = blockno * flash_block_size ;
  for (i = 0 ; i < flash_block_size ; i += 4) {
    w = 0 ;
    for (j = 0 ; j < 4 ; j++) {
      if (data == 0xff) data = 0 ;
      w |= (data << (j * 8)) ;
      data++ ;
    }
    write_flash_dword(offset, w) ;
    offset += 4 ;
  }

  PutString("Reading test pattern\n\r") ;
  data = 0 ;
  offset = blockno * flash_block_size ;
  for (i = 0 ; i < flash_block_size ; i += 4) {
    w = 0 ;
    for (j = 0 ; j < 4 ; j++) {
      if (data == 0xff) data = 0 ;
      w |= (data << (j * 8)) ;
      data++ ;
    }
    if (read_flash_dword(offset) != w) {
      ecount++ ;
      if (ecount < 5) {
	PutString("ERROR: test pattern does not match (offset = ") ;
	PutHex(offset) ; PutString("\n\r") ;
	PutString("    Value is ") ; PutHex(read_flash_dword(offset)) ; 
	PutString(", should be ") ; PutHex(w) ; PutString("\n\r") ;
      }
      return -1 ;
    }
    offset += 4 ;
  }

  PutString("Flash test worked\n\r") ;
  return 0 ;
}

/* ------------------------ M A I N ---------------------------- */
int flashTests(struct testResults *pResults)
{
  int blockno ;

  /* tell the world just who we are */
  PutString("Flash Tests [v1.0]\n\r");

  /* try and find the flash */
  PutString("Searching for flash device\n\r") ;
  if (check_flash_id((void *)FLASH_BASE) == TRUE) {
    pResults->flashBase = (unsigned int)FLASH_BASE ;
    flash_base = (unsigned char *)FLASH_BASE ;
    pResults->flashBlocks = NUMBER_OF_BLOCKS ;
    pResults->flashBlockSize = flash_block_size ;
    PutString("Flash found at ") ; 
    PutHex((unsigned int)pResults->flashBase) ; 
    PutString(" (") ; 
    PutHex(pResults->flashBlocks) ;
    PutString(" blocks of size ") ;
    PutHex(flash_block_size) ;
    PutString("\n\r") ;
  } else {
    PutString("Failed to find flash device\n\r") ;
    pResults->flashBase = 0 ;
    pResults->flashBlocks = 0 ;
    pResults->flashBlockSize = 0 ;
    return -1 ;
  }

  /* initialize */
  header = &staticHeader ;

  /* Work out how the blocks are currently being used 
     and list their contents. */
  scan_blocks() ;
  list_blocks() ;

  /* Find a spare block to play with */
  for (blockno = 2; blockno < NUMBER_OF_BLOCKS; blockno++) 
    if (!BLOCK_IN_USE(blockno)) break ;
  if (blockno >= NUMBER_OF_BLOCKS) {
    PutString("ERROR: no spare block to test with\n") ;
    return -1 ;
  }

  /* test that block */
  test_block(blockno) ;

  return 0 ;
}

