; EBSA285 manufacturing/power on self tests: memory code
; 
; History:	
; 
;
		KEEP
; --------------------------------------------------------------------------------
		IMPORT	SetLEDs
	
; --------------------------------------------------------------------------------
; test code
		EXPORT	TestMemory
		IMPORT	PrintString
		IMPORT	PrintHexWord
		IMPORT	SetLEDs
	
; --------------------------------------------------------------------------------
; include files
		INCLUDE platform.s
		GET	target.s

		AREA	SelfTests_Memory,CODE,READONLY
		LTORG

; --------------------------------------------------------------------------------
; Memory testing macros
; Macros for store multiple and load multiple tests
; These are macros, rather than functions, because the length of an
; STM or LDM is in the instruction.  The only way to write them as
; functions would be to write the STM or LDM instruction to memory
; before executing it (or using case statements on length).

		MACRO
$label		TESTSTM	$count
	;; Create a string for the number of the last register
		LCLA	renda
		LCLS	rend
renda		SETA	$count+8
	;; The following makes the string decimal instead of hex
 [ (renda>9)		
rend		SETS	(:STR:(renda+6)):RIGHT:2
 |
rend		SETS	(:STR:(renda)):RIGHT:1
 ]
$label
	;; Fill the test region with data
		SUB	r9,r0,#32
		LDR	r7,=&1234ABCD
		ADD	r1,r0,#(($count*4)+32)
01		STR	r7,[r9],#4
		CMP	r9,r1
		BNE	%B01
		MOV	r9,#0
		MOV	r10,#4
		MOV	r11,#8
		MOV	r12,#12
		MOV	r13,#16
	;; Do the store multiple
		STMIA	r0,{r9-r$rend}
	;; Test the region to which the store multiple wrote
		MOV	r7,#0
01		LDR	r1,[r0,r7]		
		CMP	r7,r1
		BNE	%F02
		ADD	r7,r7,#4
		CMP	r7,#($count*4)
		BNE	%B01
	;; Test the guard regions
		LDR	r9,=&1234abcd
		LDR	r7,=-32
01		LDR	r1,[r0,r7]
		CMP	r1,r9
		BNE	%F02
		ADD	r7,r7,#4
		CMP	r7,#0
		BNE	%B01
		MOV	r7,#$count*4
01		LDR	r1,[r0,r7]
		CMP	r1,r9
		BNE	%F02
		ADD	r7,r7,#4
		CMP	r7,#(($count*4)+32)
		BNE	%B01
	;; If we got here it all worked !
		B	%F04
	;; Error case r7 contains (possibly negative) offset, r1 the value read
02		MOV	r8,lr
		MOV	r14,r1
		MOV	r6,r0
		LDR	r0,=STMerror
		BL	PrintString
		MOV	r0,r6
		BL	PrintHexWord
		LDR	r0,=STMsize
		BL	PrintString
		MOV	r0,#$count
		BL	PrintHexWord
		LDR	r0,=STMOffset
		BL	PrintString
		MOV	r0,r7
		BL	PrintHexWord
		LDR	r0,=ValueRead
		BL	PrintString
		MOV	r0,r14
		BL	PrintHexWord
 [ :DEF: HW_DEBUG	
	;; Repeat the store multiple in a loop for h/w debug
01		STMIA	r0,{r9-r$rend}
		B	%B01
 |
	;; Continue with other tests
		LDR	r0,=TestFailed
		BL	PrintString
		MOV	lr,r8
		B	LDMtests
 ]
	;; Since this is a large macro put a literal pool somewhere convenient
		LTORG
	;; Exit from macro
04
		MEND

; Macro for load multiple tests; r0 must contain the base address on entry
		MACRO
$label		TESTLDM	$count
	;; Create a string for the number of the last register
		LCLA	renda
		LCLS	rend
renda		SETA	$count+8
	;; The following makes the string decimal instead of hex
 [ (renda>9)		
rend		SETS	(:STR:(renda+6)):RIGHT:2
 |
rend		SETS	(:STR:(renda)):RIGHT:1
 ]
	;; Fill memory with test values
$label		MOV	r7,#-32
01		STR	r7,[r0,r7]
		ADD	r7,r7,#4
		CMP	r7,#(($count*4)+32)
		BNE	%B01
	;; Put expected values in registers. This is not a processor test so I can assume
	;; that something will be written to the registers by a LDM
		MOV	r9,#0
		MOV	r10,#4
		MOV	r11,#8
		MOV	r12,#12
		MOV	r13,#16
	;; Do the LDM
		LDMIA	r0,{r9-r$rend}
	;; Test that the registers are correct
		MOV	r7,#1
		MOV	r1,r9
		CMP	r1,#0
		BNE	%F02
		MOV	r7,#2
		MOV	r1,r10
		CMP	r1,#4
		BNE	%F02
		MOV	r7,#3
		MOV	r1,r11
		CMP	r1,#8
		BNE	%F02
		MOV	r7,#4
		MOV	r1,r12
		CMP	r1,#12
		BNE	%F02
		MOV	r7,#5
		MOV	r1,r13
		CMP	r1,#16
		BNE	%F02
	;; Test for memory corruption (special request from Neal!)
		MOV	r7,#-32
01		LDR	r9,[r0,r7]
		CMP	r9,r7
		BNE	%F03
		ADD	r7,r7,#4
		CMP	r7,#(($count*4)+32)
		BNE	%B01
		B	%F05
	;; Error handling for register corruption 
	;; r7=relative word number, r1=value in register
02		MOV	r8,lr
		MOV	r14,r1
		MOV	r6,r0
		LDR	r0,=LDMRegError
		BL	PrintString
		MOV	r0,r6
		BL	PrintHexWord
		LDR	r0,=STMsize
		BL	PrintString
		MOV	r0,#$count
		BL	PrintHexWord
		LDR	r0,=RegNo
		BL	PrintString
		MOV	r0,r7
		BL	PrintHexWord
		LDR	r0,=ValueRead
		BL	PrintString
		MOV	r0,r14
		BL	PrintHexWord		
		B	%F01
	;; Error handling for memory corruption
	;; r7=offset (possibly negative) r1=value in memory
03		MOV	r8,lr
		MOV	r14,r1
		MOV	r6,r0
		LDR	r0,=LDMMemError
		BL	PrintString
		MOV	r0,r6
		BL	PrintHexWord
		LDR	r0,=STMsize
		BL	PrintString
		MOV	r0,#$count
		BL	PrintHexWord
		LDR	r0,=STMOffset
		BL	PrintString
		MOV	r0,r7
		BL	PrintHexWord
		LDR	r0,=ValueRead
		BL	PrintString
		MOV	r0,r14
		BL	PrintHexWord		
 [ :DEF: HW_DEBUG	
01		LDMIA	r6,{r9-r$rend}
		B	%B01
 |
01		LDR	r0,=TestFailed
		BL	PrintString
		B	MemTestsDone
		MOV	lr,r8
 ]

		LTORG
05	
		MEND
; --------------------------------------------------------------------------------
; Test the memory.
;
; Memtests runs the full set of memory tests on an area of memory. These are
; destructive so there must be nothing valuable in the memory. On entry r8
; contains the start of the area to be tested and r6 the end. Note that
; r0 to r5 may be corrupted by the printing functions.
	
TestMemory
	
; Do a simple memory test for storing data and aliased words

		MOV	r7,lr
		LDR	r0,=Test1
		BL	PrintString
		MOV	lr,r7
	;;  fill each word of memory with its address
		MOV	r7,r8
01		STR	r7,[r7]
		ADD	r7,r7,#4
		CMP	r7,r6
		BNE	%B01

	;; read back the data
		MOV	r7,r8
		MOV	r10,#20 ; Max errors before aborting
01		LDR	r9,[r7]
		CMP	r9,r7
		BLNE	MemError
		ADD	r7,r7,#4
		CMP	r7,r6
		BNE	%B01
		MOV	r7,lr
		CMP	r10,#20
		LDREQ	r0,=TestPassed
		LDRNE	r0,=TestFailed
		BL	PrintString
		MOV	r10,#20
;;; Test 2 - Make sure upper half of each word works
;;  fill each word of memory with its address
		LDR	r0,=Test2
		BL	PrintString
		MOV	lr,r7
		MOV	r7,r8
01		MOV	r9, r7, LSL #16
		STR	r9,[r7]
		ADD	r7,r7,#4
		CMP	r7,r6
		BNE	%B01

	;; read back the data
		MOV	r7,r8
01		LDR	r9,[r7]
		CMP	r9,r7, LSL #16
		BLNE	MemError
		ADD	r7,r7,#4
		CMP	r7,r6
		BNE	%B01
		MOV	r7,lr
		LDREQ	r0,=TestPassed
		LDRNE	r0,=TestFailed
		BL	PrintString
		MOV	r10,#20

;;; Test 3 - Byte writes and reads
		LDR	r0,=Test3
		BL	PrintString
		MOV	lr,r7
		MOV	r7,r8
		MOV	r12,r8
		AND	r12,r12,#&FF
01		CMP	r12,#&FF
		MOVEQ	r12,#0		; Avoid writing the same value to the most likely aliases
		STRB	r12,[r7],#1
		ADD	r12,r12,#1
		CMP	r7,r6
		BNE	%B01

	;; read back the data
		MOV	r7,r8
		MOV	r12,r8
		AND	r12,r12,#&FF
01		CMP	r12,#&FF
		MOVEQ	r12,#0
		LDRB	r9,[r7]
		CMP	r9,r12
		BLNE	MemError
		ADD	r12,r12,#1
		ADD	r7,r7,#1
		CMP	r7,r6
		BNE	%B01
		MOV	r7,lr
		LDREQ	r0,=TestPassed
		LDRNE	r0,=TestFailed
		BL	PrintString
		MOV	r10,#20
;;; Test 4 - read back the same data as words
		LDR	r0,=Test4
		BL	PrintString
		MOV	lr,r7
		MOV	r7,r8
		MOV	r12,r8
		AND	r12,r12,#&FF
	;; Loop through memory
01
	;; Set up the expected value in r13, note little-endian
		CMP	r12,#&FF
		MOVEQ	r12,#0
		MOV	r13,r12
		ADD	r12,r12,#1
		CMP	r12,#&FF
		MOVEQ	r12,#0
		ORR	r13,r13,r12, LSL #8
		ADD	r12,r12,#1
		CMP	r12,#&FF
		MOVEQ	r12,#0
		ORR	r13,r13,r12, LSL #16
		ADD	r12,r12,#1
		CMP	r12,#&FF
		MOVEQ	r12,#0
		ORR	r13,r13,r12, LSL #24
		ADD	r12,r12,#1
	;; Test the actual value
		LDR	r9,[r7]
		CMP	r9,r13
		BLNE	MemError
		ADD	r7,r7,#4
		CMP	r7,r6
		BNE	%B01
		CMP	r10,#20
		MOV	r7,lr
		BNE	MemTestsFailed
		LDREQ	r0,=TestPassed
		LDRNE	r0,=TestFailed
		BL	PrintString
	
;;; STM tests, test 1 to 5 registers at all allignments, but at only one
;;; location each.  Do not use start of memory since we want to check for
;;; corruption on either side of the region to which STM writes
		LDR	r0,=STMtestsStarting
		BL	PrintString
		MOV	lr,r7
		ADD	r0,r8,#&100
		TESTSTM	1
		ADD	r0,r8,#&104
		TESTSTM	1
		ADD	r0,r8,#&108
		TESTSTM	1
		ADD	r0,r8,#&10c
		TESTSTM	1
		ADD	r0,r8,#&100
		TESTSTM	2
		ADD	r0,r8,#&104
		TESTSTM	2
		ADD	r0,r8,#&108
		TESTSTM	2
		ADD	r0,r8,#&10c
		TESTSTM	2
		ADD	r0,r8,#&100
		TESTSTM	3
		ADD	r0,r8,#&104
		TESTSTM	3
		ADD	r0,r8,#&108
		TESTSTM	3
		ADD	r0,r8,#&10c
		TESTSTM	3
		ADD	r0,r8,#&100
		TESTSTM	4
		ADD	r0,r8,#&104
		TESTSTM	4
		ADD	r0,r8,#&108
		TESTSTM	4
		ADD	r0,r8,#&10c
		TESTSTM	4
		ADD	r0,r8,#&100
		TESTSTM	5
		ADD	r0,r8,#&104
		TESTSTM	5
		ADD	r0,r8,#&108
		TESTSTM	5
		ADD	r0,r8,#&10c
		TESTSTM	5
		MOV	r7,lr
		LDR	r0,=TestPassed
		BL	PrintString
		MOV	lr,r7
LDMtests
;;; LDM tests, similar to STM tests
		MOV	r7,lr
		LDR	r0,=LDMtestsStarting
		BL	PrintString
		MOV	lr,r7
		ADD	r0,r8,#&100
		TESTLDM	1
		ADD	r0,r8,#&104
		TESTLDM	1
		ADD	r0,r8,#&108
		TESTLDM	1
		ADD	r0,r8,#&10c
		TESTLDM	1
		ADD	r0,r8,#&100
		TESTLDM	2
		ADD	r0,r8,#&104
		TESTLDM	2
		ADD	r0,r8,#&108
		TESTLDM	2
		ADD	r0,r8,#&10c
		TESTLDM	2
		ADD	r0,r8,#&100
		TESTLDM	3
		ADD	r0,r8,#&104
		TESTLDM	3
		ADD	r0,r8,#&108
		TESTLDM	3
		ADD	r0,r8,#&10c
		TESTLDM	3
		ADD	r0,r8,#&100
		TESTLDM	4
		ADD	r0,r8,#&104
		TESTLDM	4
		ADD	r0,r8,#&108
		TESTLDM	4
		ADD	r0,r8,#&10c
		TESTLDM	4
		ADD	r0,r8,#&100
		TESTLDM	5
		ADD	r0,r8,#&104
		TESTLDM	5
		ADD	r0,r8,#&108
		TESTLDM	5
		ADD	r0,r8,#&10c
		TESTLDM	5
		MOV	r7,lr
		LDR	r0,=TestPassed
		BL	PrintString
	;; All done
MemTestsDone	
		LDR	r0, =0
		MOV	pc,r7

MemError	MOV	r11,lr
		LDR	r0,=MemErrorAt
		BL	PrintString
		MOV	r0,r7
		BL	PrintHexWord
		LDR	r0,=ValueRead
		BL	PrintString
		MOV	r0,r9
		BL	PrintHexWord
		SUBS	r10,r10,#1
		MOVNE	pc, r11
		LDR	r0,=TooMany
		BL	PrintString
	;; Drop through into memory test failure code
MemTestsFailed  LDR	r0,=AbortingTests
		BL	PrintString
MemTestsFailedLoop
		LDR	r0, =(RED_LED+YELLOW_LED)
		BL	SetLEDs
		LDR	r0,=0
		BL	SetLEDs
		B	MemTestsFailedLoop
	

TestPassed	DCB	"Test Passed",&a,&d,0
TestFailed	DCB	"*** BOARD FAILED TEST ***",&a,&d,0
Test1		DCB	"Word write each word's address to itself",&a,&d,0
Test2		DCB	"Word write each word's address to its top halfword",&a,&d,0
Test3		DCB	"Byte write and read each byte; contents of each byte should be address mod 255",&a,&d,0
Test4		DCB	"Word read the data written by the previous test",&a,&d,0
MemErrorAt	DCB	&a,&d,"Error at ",0
ValueRead	DCB	" value read ",0
STMtestsStarting DCB	"Store multiple tests starting",&a,&d,0
STMerror	DCB	"Store multiple error, base address is ",0
STMOffset	DCB	", offset ",0
STMsize		DCB	", number of registers ",0
LDMtestsStarting DCB	"Load multiple tests starting",&a,&d,0
LDMMemError	DCB	"Load multiple corrupted memory, base address is ",0
LDMRegError	DCB	"Load multiple read wrong values, base address is ",0
LDMdone		DCB	"Load multiple tests complete",&a,&d,0
RegNo		DCB	" relative register number ",0
TooMany		DCB	&a,&d,"*** Too Many Errors ****",&a,&d,0
AbortingTests	DCB	&a,&d
		DCB	"*********************************************",&a,&d
		DCB	"*** Aborting tests due to memory problems ***",&a,&d	
		DCB	"*********************************************",&a,&d,0
OneSIMM		DCB	"One ",0
TwoSIMM		DCB	"Two ",0
DRAMisEDO	DCB	"EDO DRAM SIMM fitted (SIMM size is",0
DRAMisBEDO	DCB	"Burst EDO DRAM SIMMs fitted (SIMM size is",0
SIMMsize	DCB	" Mbytes)",&a,&d,0
NoDRAMFound	DCB	"*** No DRAM SIMMs fitted!!! ***",&a,&d,0

		END


