;/*****************************************************************************
; 
;       Copyright  1997 Digital Equipment Corporation,
;                       Maynard, Massachusetts.
; 
;                        All Rights Reserved
; 
;Permission to use, copy, modify, and distribute this software and its 
;documentation for any purpose and without fee is hereby granted, provided  
;that the copyright notice and this permission notice appear in all copies  
;of software and supporting documentation, and that the name of Digital not  
;be used in advertising or publicity pertaining to distribution of the software 
;without specific, written prior permission. Digital grants this permission 
;provided that you prominently mark, as not part of the original, any 
;modifications made to this software or documentation.
; 
;Digital Equipment Corporation disclaims all warranties and/or guarantees  
;with regard to this software, including all implied warranties of fitness for 
;a particular purpose and merchantability, and makes no representations 
;regarding the use of, or the results of the use of, the software and 
;documentation in terms of correctness, accuracy, reliability, currentness or
;otherwise; and you rely on the software, documentation and results solely at 
;your own risk. 
;
;******************************************************************************
; 
;  PowerOn Selftest code for the SA-110 based EBSA285 evaluation board.
; 
;	$Id: post.s,v 1.11 1997/09/16 12:28:24 rusling Exp quinn $
;
;******************************************************************************/
;
		KEEP
; --------------------------------------------------------------------------------
; support routines (all in io.s) 
		IMPORT	ResetCOMPort
		IMPORT	PrintString
		IMPORT	PrintHexWord
		IMPORT	GetByte
		IMPORT	PutByte
		IMPORT	SetLEDs
		IMPORT	Pause
	
; --------------------------------------------------------------------------------
; test code
		IMPORT	InitializeMemory
		IMPORT	TestMemory

; --------------------------------------------------------------------------------
; C code
		IMPORT	main

; --------------------------------------------------------------------------------
; include files
		INCLUDE platform.s
		GET	target.s

; --------------------------------------------------------------------------------
; exports
		EXPORT	__main
		EXPORT	EmbeddedCCodeStart
		EXPORT	EmbeddedCCodeEnd

; --------------------------------------------------------------------------------
; Define a data area for various statistics and stacks; note that this is at the
; end of the on board SDRAM.
;
TESTERRORS	EQU	(SZ_8M - 0x1000) - 0x4
FLASHIMAGES	EQU	(SZ_8M - 0x1000) - 0x8
SSRAMSIZE	EQU	(SZ_8M - 0x1000) - 0xc
DRAMSIZE	EQU	(SZ_8M - 0x1000) - 0x10
DRAMTYPE	EQU	(SZ_8M - 0x1000) - 0x14
CBT_TYPE	EQU	(SZ_8M - 0x1000) - 0x18 ; 0 - ARCH; 1 - OS
STACK_BASE	EQU	(SZ_8M - 0x1000) - 0x1c
ABORT_STACK_BASE EQU	(SZ_8M - 0x1000) - 0x21c
IRQ_STACK_BASE	EQU	(SZ_8M - 0x1000) - 0x41c
FIQ_STACK_BASE	EQU	(SZ_8M - 0x1000) - 0x61c

; --------------------------------------------------------------------------------
		AREA	SelfTests,CODE,READONLY
;	 Start of executable code
		ENTRY
		LTORG
__main
__SelfTests
; --------------------------------------------------------------------------------
; Initialize
; Loading pc with __HiReset makes sure we're running from
; real rom, rather than a shadowed address. The 1st write after
; reset switches the memory map for normal operation.
	
	LDR	pc,=__HiReset
__HiReset
	STR	r0,[r0,-r0]

;	Initialize the X bus, then we can see the UARTs and the LEDs.
		INIT_XBUS r0, r1
	
;	Do something that the user can see.  Turn the leds all off, wait a 
;	moment and then turn them all on, wait a moment and turn them all
;	off (again).
		LDR	r0,=0
		BL	SetLEDs
		BL	Pause

		LDR	r0,=(RED_LED+YELLOW_LED+GREEN_LED)
		BL	SetLEDs
		BL	Pause

		LDR	r0,=0
		BL	SetLEDs

;	Tell the user that we have been initialized and print a nice banner
		BL	ResetCOMPort
		LDR	r0,=TestsStarted
		BL	PrintString

;	Turn on Clock Switching
		LDR	r0,=EnableCS
		BL	PrintString
		WRCP15_EnableClockSW	r0
		LDR	r0,=Done
		BL	PrintString
	
; ------------------------------------------------------------------------------
; Memory initialization and testing
		LDR	r0,=Initialising_DRAM
		BL	PrintString
;	INIT_RAM returns the size of memory in the third argument (for us, r5)
;	It uses a hell of a lot more registers though.
		INIT_RAM r0, r1, r5
		MOV	r0, r5

;	Check for failures and then save the results and light the leds.
		CMP	r0,#0
		MOVEQ	r6,#1 
		BEQ	EarlyFailure

		MOV	r6,r0		; End of test area is memory size
; ------------------------------------------------------------------------------
;	Turn on the I-Cache
		LDR	r0,=EnableICache
		BL	PrintString
		LDR	r0,=IC_ON
		WRCP15_Control	r0
		LDR	r0,=Done
		BL	PrintString
	
		LDR	r0,=GREEN_LED
		BL	SetLEDs
; ------------------------------------------------------------------------------

;	Do the memory tests (r6 = end of test area, r8 = start)
;	If these fail then it never returns.
		LDR	r8,=0
		LDR	r0,=DRAM_Test_Start
		BL	PrintString
		BL	TestMemory
		LDR	r0,=DRAM_Test_End
		BL	PrintString

;	Save the memory size in the now tested memory
	
		LDR	r1,=DRAMSIZE			; and save the result
		STR	r6,[r1]
		LDR	r0,=GREEN_LED
		BL	SetLEDs
	
;	Tell the world how much memory we have
		LDR	r1,=DRAMSIZE
		LDR	r7,[r1]
		LDR	r0,=DRAMSizeIs
		BL	PrintString
		MOV	r0,r7
		BL	PrintHexWord
		LDR	r0,=EndLine
		BL	PrintString


;	Initialise the statistics
	
		LDR	r0,=TESTERRORS
		MOV	r1,#0	        ; Any error before here is either fatal or only known to the
					; user (e.g. COM1 printing fails).
		STR	r1,[r0]

; --------------------------------------------------------------------------------
; Set up a stack to make the remaining tests easier (can't be done before memory 
; tests because they use every register)
		LDR	r13,=STACK_BASE

;	Set up Abort and interrupt stacks
		MRS	r0,CPSR
		MOV	r1,#0xD7 ; Abort mode, I don't want any interrupts for now
		MSR	CPSR_c,r1
		LDR	r13,=ABORT_STACK_BASE
		MOV	r1,#0xD2 ; IRQ mode, I don't want any interrupts for now
		MSR	CPSR_c,r1
		LDR	r13,=IRQ_STACK_BASE
		MOV	r1,#0xD1 ; FIQ mode, I don't want any interrupts for now
		MSR	CPSR_c,r1
		LDR	r13,=FIQ_STACK_BASE
		MSR	CPSR_c,r0 ; Back to the original mode

; --------------------------------------------------------------------------------
; Now get into C code, returns 0 if successful, otherwise failure
; Use ARM Procedure Call Standard nomenclature
		LDR	a2,=DRAMSIZE
		LDR	a1,[a2]
		BL	main		; Call main, falling through to
					; exit on return.
		CMP	r0,#0
		BNE	BadTestEnd
		
__exit					; exit

; --------------------------------------------------------------------------------
; End of test (good)
GoodTestEnd
		LDR	r0,=TestsFinished
		BL	PrintString

;	Loop forever...
GoodLoop
		LDR	r0, =GREEN_LED
		BL	SetLEDs
		BL	Pause

		LDR	r0,=0
		BL	SetLEDs
		BL	Pause

		B	GoodLoop
	
; --------------------------------------------------------------------------------
; End of test (early failure)
EarlyFailure	LDR	r0,=EarlyFailureMsg
		BL	PrintString
		LDR	r0,=TestsFailedMsg
		BL	PrintString	
		MOV	r0,r6
		BL	PrintHexWord
		LDR	r0,=FailuresMsg
		BL	PrintString
BadTestEnd
		LDR	r0,=TestsFinished
		BL	PrintString

;	Loop forever...
BadLoop
		LDR	r0, =RED_LED
		BL	SetLEDs
		BL	Pause
	
		LDR	r0,=0
		BL	SetLEDs
		BL	Pause
	
		B	BadLoop

	
; 
; Messages
;
		LTORG	
TestsStarted	DCB	&a,&d
		DCB	"==========================================================",&a,&d
		DCB	"EBSA285 PowerOn Selftests",&a,&d
		DCB	"==========================================================",&a,&d,0
EnableCS	DCB	"Enabling Clock Switching",0
EnableICache	DCB	"Enabling I-Cache",0
Done		DCB	"...Done",&a,&d,0
TestingRefreshTimer DCB	"*** Testing Refresh Timer ***",&a,&d,0
RefreshTestDone DCB	"*** Refresh Timer Tests Complete ***",&a,&d,0
Initialising_DRAM DCB	"*** Initialising DRAM ***",&a,&d,0
BedoDRAMFound	DCB	"Either no DRAM is fitted or Burst EDO DRAMs are fitted",&a,&d,0
EdoDRAMFound	DCB	"EDO DRAMs are fitted",&a,&d,0
DRAMAlreadyRunning DCB	"DRAM already running so type cannot be determined",&a,&d,0
Sizing_DRAM	DCB	"Sizing DRAM",&a,&d,0
DRAMSizeIs	DCB	"DRAM size is ",0
DRAM_Test_Start DCB	"*** Starting DRAM tests ***",&a,&d,0
DRAM_Test_End	DCB	"*** DRAM tests complete ***",&a,&d,0
TestingOtherTimers DCB	"*** Testing PIT counters 1 and 2 ***",&a,&d,0
OtherTimersTested DCB	"*** Tests for PIT counters 1 and 2 complete ***",&a,&d,0
TestingAbort	DCB	"*** Testing trick box aborts ***",&a,&d,0
AbortTested	DCB	"*** Trick box abort tests complete ***",&a,&d,0
TestingTcycle	DCB	"*** Testing trick box cycle counters ***",&a,&d,0
TcycleTested	DCB	"*** Trick box cycle counters complete ***",&a,&d,0
TestingFlashControl	DCB	"*** Testing flash control register access ***",&a,&d,0
FlashControlTested	DCB	"*** Flash control register access test complete ***",&a,&d,0
GettingFrequencies	DCB	"*** Calculating processor and bus speed",&a,&d,0
TestingComPorts	DCB	"*** Testing COM ports ***",&a,&d,0
ComPortsTested	DCB	"*** COM port tests complete ***",&a,&d,0
TestingFlash	DCB	"*** Testing Flash ***",&a,&d,0
FlashTested	DCB	"*** Flash tests complete ***",&a,&d,0
ProgramWithErrors DCB	"Some tests failed! Skip programming the Flash (Y/n)?",&a,&d
		DCB	">>",0
ProgrammingFlash DCB	"*** Programming images into Flash ***",&a,&d,0
FlashProgrammed DCB	"*** Flash programming complete ***",&a,&d,0
JumperSettings	DCB	"Jumper settings:",&a,&d,0
Jumper9_10	DCB	" J2 pins 9-10 ",0
Jumper11_12	DCB	" J2 pins 11-12 ",0
Jumper13_14	DCB	" J2 pins 13-14 ",0
Jumper15_16	DCB	" J2 pins 15-16 ",0
Fitted		DCB	"Fitted",&a,&d,0
NotFitted	DCB	"Not Fitted",&a,&d,0
SummaryCorrect	DCB	"Is this summary correct (y/N)?",&a,&d
		DCB	">>",0
			
TestsPassedMsg	DCB	&a,&d,"*** TESTS PASSED ***",&a,&d,&a,&d,0
TestsFailedMsg	DCB	&a,&d,"*** TESTS FAILED ***",&a,&d
		DCB	"*** TESTS FAILED ***",&a,&d
		DCB	"*** TESTS FAILED ***",&a,&d,&a,&d,0
FailuresMsg	DCB	" tests failed",&a,&d,0
SummaryMsg	DCB	"Summary of board configuration detected",&a,&d
		DCB	"=======================================",&a,&d
		DCB	"CPU Identification is ",0
EarlyFailureMsg DCB	"A memory or refresh timer test failed; not continuing",&a,&d,0

TestsFinished	DCB	&a,&d
		DCB	"==========================================================",&a,&d
		DCB	"Selftests complete, change boot selection before rebooting",&a,&d
		DCB	"==========================================================",&a,&d,0
	
;
; Usefull pieces of lines
;
EndLine		DCB	&a,&d,0

		ALIGN	4
EmbeddedCCodeStart
		INCLUDE	tests.dcb
EmbeddedCCodeEnd
		END


